import { ElementRef, EventEmitter } from '@angular/core';
import { OpenViduDirective, CameraAccessEvent, ErrorEvent, MessageEvent, ParticipantEvent, RoomConnectedEvent, StreamEvent } from '../openvidu.directive';
import { Connection, Session, Stream } from 'openvidu-browser';
export declare enum ConnectionState {
    NOT_CONNECTED = 0,
    CONNECTED_TO_SERVER = 1,
    CONNECTED_TO_ROOM = 2,
    REQUESTING_CAMERA_ACCESS = 3,
    CAMERA_ACCESS_GRANTED = 4,
    CAMERA_ACCESS_DENIED = 5,
}
export interface ToolbarOption {
    label?: string;
    icon: string;
    onClick?: Function;
}
export declare abstract class OpenViduInternalComponent {
    wsUrl: string;
    sessionId: string;
    participantId: string;
    apiKey: string;
    token: string;
    onServerConnected: EventEmitter<void>;
    onErrorServer: EventEmitter<any>;
    onRoomConnected: EventEmitter<void>;
    onErrorRoom: EventEmitter<any>;
    onRoomClosed: EventEmitter<void>;
    onLostConnection: EventEmitter<any>;
    onParticipantJoined: EventEmitter<any>;
    onParticipantLeft: EventEmitter<any>;
    onErrorMedia: EventEmitter<any>;
    onLeaveRoom: EventEmitter<void>;
    onNewMessage: EventEmitter<any>;
    onCustomNotification: EventEmitter<any>;
    openviduApi: OpenViduDirective;
    userMessage: string;
    isFullscreen: boolean;
    micOffParticipants: Connection[];
    camOffParticipants: Connection[];
    mainStream: Stream;
    streams: Stream[];
    chatMessages: any[];
    protected session: Session;
    protected participants: {
        [id: string]: Connection;
    };
    protected myCamera: Stream;
    ConnectionState: typeof ConnectionState;
    connectionUiState: ConnectionState;
    sendCustomNotification(obj: any, callback: any): void;
    leaveRoom(callLeaveRoom?: boolean): void;
    handleOnUpdateMainSpeaker(streamEvent: StreamEvent): void;
    handleOnServerConnected(): void;
    handleOnErrorServer(errorEvent: ErrorEvent): void;
    handleOnRoomConnected(roomConnectedEvent: RoomConnectedEvent): void;
    handleOnErrorRoom(): void;
    handleOnCameraAccessChange(cameraEvent: CameraAccessEvent): void;
    handleOnRoomClosed(): void;
    handleOnLostConnection(): void;
    handleOnParticipantJoined(participantEvent: ParticipantEvent): void;
    handleOnParticipantLeft(participantEvent: ParticipantEvent): void;
    handleOnErrorMedia(errorEvent: ErrorEvent): void;
    handleOnStreamAdded(streamEvent: StreamEvent): void;
    handleOnStreamRemoved(streamEvent: StreamEvent): void;
    handleOnLeaveRoom(): void;
    handleOnNewMessage(messageEvent: MessageEvent): void;
    handleOnCustomNotification(obj: any): void;
    protected setUserMessage(msg: string): void;
    /**
     * Function to resize the video elements based on the container
     * @return Object {width?: number, height?: number, error?: boolean}
     */
    protected auxResizeStreamsManually(container: ElementRef, boxes: ElementRef[]): {
        width?: number;
        height?: number;
        error?: boolean;
    };
}
