"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
// OpenVidu Directive
var openvidu_directive_1 = require("../openvidu.directive");
var ConnectionState;
(function (ConnectionState) {
    ConnectionState[ConnectionState["NOT_CONNECTED"] = 0] = "NOT_CONNECTED";
    ConnectionState[ConnectionState["CONNECTED_TO_SERVER"] = 1] = "CONNECTED_TO_SERVER";
    ConnectionState[ConnectionState["CONNECTED_TO_ROOM"] = 2] = "CONNECTED_TO_ROOM";
    ConnectionState[ConnectionState["REQUESTING_CAMERA_ACCESS"] = 3] = "REQUESTING_CAMERA_ACCESS";
    ConnectionState[ConnectionState["CAMERA_ACCESS_GRANTED"] = 4] = "CAMERA_ACCESS_GRANTED";
    ConnectionState[ConnectionState["CAMERA_ACCESS_DENIED"] = 5] = "CAMERA_ACCESS_DENIED";
})(ConnectionState = exports.ConnectionState || (exports.ConnectionState = {}));
var OpenViduInternalComponent = (function () {
    function OpenViduInternalComponent() {
        // Outputs
        this.onServerConnected = new core_1.EventEmitter();
        this.onErrorServer = new core_1.EventEmitter();
        this.onRoomConnected = new core_1.EventEmitter();
        this.onErrorRoom = new core_1.EventEmitter();
        this.onRoomClosed = new core_1.EventEmitter();
        this.onLostConnection = new core_1.EventEmitter();
        this.onParticipantJoined = new core_1.EventEmitter();
        this.onParticipantLeft = new core_1.EventEmitter();
        this.onErrorMedia = new core_1.EventEmitter();
        this.onLeaveRoom = new core_1.EventEmitter();
        this.onNewMessage = new core_1.EventEmitter();
        this.onCustomNotification = new core_1.EventEmitter();
        // Flags for HTML elements
        this.userMessage = '';
        this.isFullscreen = false;
        // Participnats who have enabled/disabled their mic
        this.micOffParticipants = [];
        // Participnats who have enabled/disabled their cam
        this.camOffParticipants = [];
        // Rest of peers
        this.streams = [];
        // Chat
        this.chatMessages = [];
        // Participants
        this.participants = {};
        // Connection state
        this.ConnectionState = ConnectionState;
        this.connectionUiState = ConnectionState.NOT_CONNECTED;
    }
    /*---------------------*/
    /* GUI RELATED METHODS */
    /*---------------------*/
    OpenViduInternalComponent.prototype.sendCustomNotification = function (obj, callback) {
        this.openviduApi.sendCustomNotification(obj, callback);
    };
    OpenViduInternalComponent.prototype.leaveRoom = function (callLeaveRoom) {
        // Reset
        this.mainStream = null;
        this.streams = [];
        this.chatMessages = [];
        this.session = null;
        this.participants = {};
        if (callLeaveRoom !== false) {
            this.openviduApi.leaveRoom();
        }
    };
    /*------------------------*/
    /* HANDLE OPENVIDU EVENTS */
    /*------------------------*/
    OpenViduInternalComponent.prototype.handleOnUpdateMainSpeaker = function (streamEvent) {
        // Check if stream exists
        if (this.streams.indexOf(streamEvent.stream) < 0) {
            this.streams.push(streamEvent.stream);
        }
        // Set main stream
        this.mainStream = streamEvent.stream;
        // Check if participant exists
        if (!this.participants[this.mainStream.getParticipant().connectionId]) {
            this.participants[this.mainStream.getParticipant().connectionId] = this.mainStream.getParticipant();
        }
    };
    OpenViduInternalComponent.prototype.handleOnServerConnected = function () {
        this.connectionUiState = ConnectionState.CONNECTED_TO_SERVER;
    };
    OpenViduInternalComponent.prototype.handleOnErrorServer = function (errorEvent) {
        if (errorEvent.error) {
            this.setUserMessage(errorEvent.error.message);
        }
    };
    OpenViduInternalComponent.prototype.handleOnRoomConnected = function (roomConnectedEvent) {
        if (roomConnectedEvent && roomConnectedEvent.session) {
            this.session = roomConnectedEvent.session;
        }
        // Emit event
        this.onRoomConnected.emit();
        this.connectionUiState = ConnectionState.CONNECTED_TO_ROOM;
    };
    OpenViduInternalComponent.prototype.handleOnErrorRoom = function () {
        // Emit event
        this.onErrorRoom.emit();
    };
    OpenViduInternalComponent.prototype.handleOnCameraAccessChange = function (cameraEvent) {
        if (cameraEvent.access) {
            // All good :)
            this.handleOnStreamRemoved({
                stream: this.myCamera
            });
            this.myCamera = cameraEvent.camera;
            this.handleOnStreamAdded({
                stream: this.myCamera
            });
            this.connectionUiState = ConnectionState.CAMERA_ACCESS_GRANTED;
        }
        else if (!cameraEvent.access) {
            // No camera :(
            this.connectionUiState = ConnectionState.CAMERA_ACCESS_DENIED;
        }
    };
    OpenViduInternalComponent.prototype.handleOnRoomClosed = function () {
        this.session = null;
        // Emit event
        this.onRoomClosed.emit();
    };
    OpenViduInternalComponent.prototype.handleOnLostConnection = function () {
        if (!this.session) {
            // Emit event
            this.onLostConnection.emit({
                error: new Error('Lost connection with the server')
            });
        }
        else {
            // Emit event
            this.onLostConnection.emit();
        }
    };
    OpenViduInternalComponent.prototype.handleOnParticipantJoined = function (participantEvent) {
        var newParticipant = participantEvent.participant;
        this.participants[newParticipant.connectionId] = newParticipant;
        // Emit event
        this.onParticipantJoined.emit({
            participantId: newParticipant.connectionId
        });
    };
    OpenViduInternalComponent.prototype.handleOnParticipantLeft = function (participantEvent) {
        var oldParticipant = participantEvent.participant;
        delete this.participants[oldParticipant.connectionId];
        // Emit event
        this.onParticipantLeft.emit({
            participantId: oldParticipant.connectionId
        });
    };
    OpenViduInternalComponent.prototype.handleOnErrorMedia = function (errorEvent) {
        // Emit event
        this.onErrorMedia.emit({
            error: errorEvent.error
        });
    };
    OpenViduInternalComponent.prototype.handleOnStreamAdded = function (streamEvent) {
        var newStream = streamEvent.stream;
        if (this.streams.indexOf(newStream) < 0) {
            this.streams.push(newStream);
        }
    };
    OpenViduInternalComponent.prototype.handleOnStreamRemoved = function (streamEvent) {
        var oldStream = streamEvent.stream;
        this.streams.splice(this.streams.indexOf(oldStream), 1);
    };
    OpenViduInternalComponent.prototype.handleOnLeaveRoom = function () {
        this.session = null;
        this.participants = {};
        this.streams = [];
        this.mainStream = null;
        // Emit event
        this.onLeaveRoom.emit();
    };
    OpenViduInternalComponent.prototype.handleOnNewMessage = function (messageEvent) {
        // Fix: to get usernam
        var dataObj = JSON.parse(messageEvent.participant.data);
        // Handle message
        this.chatMessages.push({
            username: dataObj.username,
            message: messageEvent.message,
            date: new Date() // Use current date
        });
        this.onNewMessage.emit(messageEvent);
    };
    OpenViduInternalComponent.prototype.handleOnCustomNotification = function (obj) {
        if (obj.openviduType) {
            // TODO: Internal custom notification
            switch (obj.openviduType) {
                case openvidu_directive_1.OpenViduNotificationType.MIC_CHANGED:
                    if (obj.micEnabled) {
                        // Remove participant from micOffParticipant
                    }
                    else {
                        // Add participant to micOffParticipant
                    }
                    break;
                case openvidu_directive_1.OpenViduNotificationType.CAM_CHANGED:
                    if (obj.micEnabled) {
                        // Remove participant from camOffParticipant
                    }
                    else {
                        // Add participant to camOffParticipant
                    }
                    break;
            }
        }
        else {
            this.onCustomNotification.emit(obj);
        }
    };
    /*-------------------*/
    /* PROTECTED METHODS */
    /*-------------------*/
    OpenViduInternalComponent.prototype.setUserMessage = function (msg) {
        this.userMessage = msg;
    };
    /*-----------------*/
    /* PRIVATE METHODS */
    /*-----------------*/
    /**
     * Function to resize the video elements based on the container
     * @return Object {width?: number, height?: number, error?: boolean}
     */
    OpenViduInternalComponent.prototype.auxResizeStreamsManually = function (container, boxes) {
        if (!container || !boxes || boxes.length === 0)
            return { error: true };
        var maxDimensions = {
            maxW: 0,
            maxH: 0
        };
        boxes.forEach(function (box) {
            var videoEl = box.nativeElement;
            if (videoEl.videoWidth > maxDimensions.maxW) {
                if (videoEl.videoHeight > maxDimensions.maxH) {
                    maxDimensions.maxW = videoEl.videoWidth;
                    maxDimensions.maxH = videoEl.videoHeight;
                }
            }
        });
        //console.log(maxDimensions);
        var clientRect = container.nativeElement.getBoundingClientRect();
        var numElements = boxes.length;
        var width = clientRect.width;
        var height = clientRect.height;
        var area = height * width;
        var elementArea = parseInt((area / numElements) + '');
        // Calculate proportions
        var maxProportions = {
            maxW: 0,
            maxH: 0,
        };
        if (width > height) {
            // It'a horizontal rectangle
            maxProportions.maxW = maxDimensions.maxW / maxDimensions.maxH;
            maxProportions.maxH = ((maxDimensions.maxW / maxDimensions.maxH) * maxDimensions.maxH) / maxDimensions.maxW;
        }
        else if (height > width) {
            // It'a vertcal rectangle
            maxProportions.maxW = ((maxDimensions.maxH / maxDimensions.maxW) * maxDimensions.maxW) / maxDimensions.maxH;
            maxProportions.maxH = maxDimensions.maxH / maxDimensions.maxW;
        }
        else {
            // It's a square
            maxProportions.maxW = maxDimensions.maxW / maxDimensions.maxH;
            maxProportions.maxH = ((maxDimensions.maxW / maxDimensions.maxH) * maxDimensions.maxH) / maxDimensions.maxW;
        }
        //console.log(maxProportions);
        var elementWidth = parseInt(Math.sqrt(elementArea * (maxProportions.maxW / maxProportions.maxH)) + '');
        var elementHeight = parseInt(Math.sqrt(elementArea * (maxProportions.maxH / maxProportions.maxW)) + '');
        //console.log(elementWidth);
        // We now need to fit the squares. Let's reduce the square size
        // so an integer number fits the width.
        var numX = Math.ceil(width / elementWidth);
        elementWidth = width / numX;
        elementHeight = elementWidth * (maxProportions.maxH / maxProportions.maxW);
        while (numX <= numElements) {
            // With a bit of luck, we are done.
            if (Math.floor(height / elementHeight) * numX >= numElements) {
                // They all fit! We are done!
                return { width: elementWidth, height: elementHeight };
            }
            // They don't fit. Make room for one more square i each row.
            numX++;
            elementWidth = width / numX;
            elementHeight = elementWidth * (maxProportions.maxH / maxProportions.maxW);
        }
        // Still doesn't fit? The window must be very wide
        // and low.
        elementHeight = height;
        return { width: elementWidth, height: elementHeight };
    };
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], OpenViduInternalComponent.prototype, "wsUrl", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], OpenViduInternalComponent.prototype, "sessionId", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], OpenViduInternalComponent.prototype, "participantId", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], OpenViduInternalComponent.prototype, "apiKey", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], OpenViduInternalComponent.prototype, "token", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onServerConnected", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onErrorServer", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onRoomConnected", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onErrorRoom", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onRoomClosed", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onLostConnection", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onParticipantJoined", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onParticipantLeft", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onErrorMedia", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onLeaveRoom", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onNewMessage", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], OpenViduInternalComponent.prototype, "onCustomNotification", void 0);
    __decorate([
        core_1.ViewChild('openviduApi'),
        __metadata("design:type", openvidu_directive_1.OpenViduDirective)
    ], OpenViduInternalComponent.prototype, "openviduApi", void 0);
    return OpenViduInternalComponent;
}());
exports.OpenViduInternalComponent = OpenViduInternalComponent;
