"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var testing_1 = require("@angular/core/testing");
// Imports dependencies of component to be tested
var openvidu_module_1 = require("./openvidu.module");
// Component to be tested
var openvidu_directive_1 = require("./openvidu.directive");
// Testing utils
var testing_helper_1 = require("./testing/testing-helper");
var TestComponent = (function () {
    function TestComponent() {
        this.serverConnected = false;
        this.messages = [];
    }
    TestComponent.prototype.onServerConnected = function () {
        // Dummy
        this.serverConnected = true;
    };
    TestComponent.prototype.onRoomConnected = function () {
        // Dummy
    };
    TestComponent.prototype.onParticipantJoined = function () {
        // Dummy
    };
    TestComponent.prototype.onParticipantLeft = function () {
        // Dummy
    };
    TestComponent.prototype.onErrorMedia = function () {
        // Dummy
    };
    TestComponent.prototype.onLeaveRoom = function () {
        // Dummy
    };
    TestComponent.prototype.onNewMessage = function (messageEvent) {
        this.messages.push(messageEvent.message);
    };
    __decorate([
        core_1.ViewChild('openviduApi'),
        __metadata("design:type", openvidu_directive_1.OpenViduDirective)
    ], TestComponent.prototype, "openviduApi", void 0);
    TestComponent = __decorate([
        core_1.Component({
            template: "\n\t<openvidu-template\n\t\t#openviduApi=\"openviduApi\"\n\t\t[wsUrl]=\"wsUrl\" [sessionId]=\"sessionId\" [participantId]=\"participantId\"\n\t\t(onServerConnected)=\"onServerConnected()\"\n\t\t(onRoomConnected)=\"onRoomConnected()\"\n\t\t(onParticipantJoined)=\"onParticipantJoined($event)\"\n\t\t(onParticipantLeft)=\"onParticipantLeft($event)\"\n\t\t(onErrorMedia)=\"onErrorMedia()\"\n\t\t(onLeaveRoom)=\"onLeaveRoom()\"\n\t\t(onNewMessage)=\"onNewMessage($event)\">\n\t\tLoading openvidu...\n\t</openvidu-template>"
        })
    ], TestComponent);
    return TestComponent;
}());
describe('AngularOpenVidu Directive', function () {
    var fixture;
    var testComponent;
    var defaultWsUrl = 'wss://127.0.0.1:8443/';
    beforeEach(testing_1.async(function () {
        // Setup the component to be tested
        testing_1.TestBed.configureTestingModule({
            imports: [
                openvidu_module_1.OpenViduModule
            ],
            declarations: [
                TestComponent
            ],
            providers: [
                { provide: testing_1.ComponentFixtureAutoDetect, useValue: true }
            ]
        })
            .compileComponents().then(function () {
            fixture = testing_1.TestBed.createComponent(TestComponent);
            fixture.detectChanges();
            testComponent = fixture.componentInstance;
        });
        jasmine.DEFAULT_TIMEOUT_INTERVAL = 10000;
    }));
    afterAll(function () {
        // Reset the testing module
        //getTestBed().resetTestingModule();
    });
    it('should create the component', function () {
        // Check that the componente has been instanciated
        expect(testComponent).toBeTruthy();
    });
    it('should trigger ngOnChanges', function () {
        // Setup spy
        spyOn(testComponent.openviduApi, 'ngOnChanges').and.callThrough();
        // No calls
        expect(testComponent.openviduApi.ngOnChanges).not.toHaveBeenCalled();
        // One call
        testComponent.sessionId = 'dummy';
        fixture.detectChanges();
        expect(testComponent.openviduApi.ngOnChanges).toHaveBeenCalled();
        // Check values
        var randomCredentials = testing_helper_1.getRandomCredentials();
        testComponent.sessionId = randomCredentials['sessionId'];
        testComponent.participantId = randomCredentials['participantId'];
        testComponent.wsUrl = defaultWsUrl;
        fixture.detectChanges();
        expect(testComponent.openviduApi.sessionId).toEqual(randomCredentials['sessionId']);
        expect(testComponent.openviduApi.participantId).toEqual(randomCredentials['participantId']);
        expect(testComponent.openviduApi.wsUrl).toEqual(defaultWsUrl);
    });
    it('should trigger onServerConnected', function (done) {
        console.log(testComponent.openviduApi.wsUrl);
        // Setup spy
        spyOn(testComponent, 'onServerConnected').and.callThrough();
        // Setup creds
        var randomCredentials = testing_helper_1.getRandomCredentials();
        testComponent.sessionId = randomCredentials['sessionId'];
        testComponent.participantId = randomCredentials['participantId'];
        testComponent.wsUrl = defaultWsUrl;
        fixture.detectChanges();
        // Check trigger
        setTimeout(function () {
            expect(testComponent.onServerConnected).toHaveBeenCalled();
            done();
        }, 1000);
    });
    it('should trigger onRoomConnected', function (done) {
        // Setup spy
        spyOn(testComponent, 'onRoomConnected').and.callThrough();
        // setup creds
        var randomCredentials = testing_helper_1.getRandomCredentials();
        testComponent.sessionId = randomCredentials['sessionId'];
        testComponent.participantId = randomCredentials['participantId'];
        testComponent.wsUrl = defaultWsUrl;
        fixture.detectChanges();
        // Check trigger
        setTimeout(function () {
            expect(testComponent.onRoomConnected).toHaveBeenCalled();
            done();
        }, 4000);
    });
    it('should trigger onLeaveRoom', function (done) {
        // Setup spy
        spyOn(testComponent, 'onLeaveRoom').and.callThrough();
        // setup creds
        var randomCredentials = testing_helper_1.getRandomCredentials();
        testComponent.sessionId = randomCredentials['sessionId'];
        testComponent.participantId = randomCredentials['participantId'];
        testComponent.wsUrl = defaultWsUrl;
        fixture.detectChanges();
        testComponent.openviduApi.leaveRoom();
        // Check trigger
        setTimeout(function () {
            expect(testComponent.onLeaveRoom).toHaveBeenCalled();
            done();
        }, 1000);
    });
    it('should trigger onNewMessage', function (done) {
        // Fix popup from openvidu-browser
        //window.alert = function(){return;};
        window.confirm = function () { return false; };
        // Setup spy
        spyOn(testComponent, 'onNewMessage').and.callThrough();
        // setup creds
        var randomCredentials = testing_helper_1.getRandomCredentials();
        testComponent.sessionId = randomCredentials['sessionId'];
        testComponent.participantId = randomCredentials['participantId'];
        testComponent.wsUrl = defaultWsUrl;
        fixture.detectChanges();
        // Check trigger
        setTimeout(function () {
            var testMessage = 'Testing message';
            testComponent.openviduApi.sendMessage(testMessage);
            // Check trigger
            setTimeout(function () {
                expect(testComponent.onNewMessage).toHaveBeenCalled();
                expect(testComponent.messages.length).toEqual(1);
                expect(testComponent.messages[0]).toEqual(testMessage);
                done();
            }, 4000);
        }, 4000);
    });
    it('should change camera', function () {
        // Setup spy
        spyOn(testComponent, 'onNewMessage').and.callThrough();
        // setup creds
        var randomCredentials = testing_helper_1.getRandomCredentials();
        testComponent.sessionId = randomCredentials['sessionId'];
        testComponent.participantId = randomCredentials['participantId'];
        testComponent.wsUrl = defaultWsUrl;
        fixture.detectChanges();
        // Check trigger
        setTimeout(function () {
            testComponent.openviduApi.camEnabled = false;
            expect(testComponent.messages.length).not.toBeTruthy(testComponent.openviduApi.camEnabled);
        }, 4000);
    });
    it('should chnage microphone', function () {
        // Setup spy
        spyOn(testComponent, 'onNewMessage').and.callThrough();
        // setup creds
        var randomCredentials = testing_helper_1.getRandomCredentials();
        testComponent.sessionId = randomCredentials['sessionId'];
        testComponent.participantId = randomCredentials['participantId'];
        testComponent.wsUrl = defaultWsUrl;
        fixture.detectChanges();
        // Check trigger
        setTimeout(function () {
            testComponent.openviduApi.micEnabled = false;
            expect(testComponent.messages.length).not.toBeTruthy(testComponent.openviduApi.micEnabled);
        }, 4000);
    });
});
