"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
// Parent component
var openvidu_internal_component_1 = require("../openvidu-internal/openvidu-internal.component");
// Angular Material
var material_1 = require("@angular/material");
// Fullscreen Service
var angular_bigscreen_1 = require("angular-bigscreen");
// i18n Labels and Messages
var openvidu_appearin_intl_1 = require("./openvidu-appearin-intl");
var OpenViduAppearinComponent = (function (_super) {
    __extends(OpenViduAppearinComponent, _super);
    function OpenViduAppearinComponent(renderer, bigScreenService, _intl) {
        var _this = _super.call(this) || this;
        _this.renderer = renderer;
        _this.bigScreenService = bigScreenService;
        _this._intl = _intl;
        // Inputs
        // To set new options in menu
        _this.toolbarOptions = [];
        // Responsive streams
        _this.streamMaxWidth = '100%';
        _this.streamMaxHeight = '100%';
        // Flags for HTML elements
        _this.welcome = true;
        _this.showChat = false;
        _this.welcome = true;
        _this.setUserMessage(_this._intl.loadingLabel);
        return _this;
    }
    OpenViduAppearinComponent.prototype.ngOnInit = function () {
        var _this = this;
        // Display message
        this.setUserMessage(this._intl.connectingLabel);
        // Set fullscreen listener
        this.bigScreenService.onChange(function () {
            // No need to check if mainElement is the one with fullscreen
            _this.isFullscreen = _this.bigScreenService.isFullscreen();
            // Fix: Resize videos
            setTimeout(function () {
                _this.resizeStreamsManually();
            }, 1000);
        });
    };
    OpenViduAppearinComponent.prototype.ngOnDestroy = function () {
        this.bigScreenService.exit();
        this.leaveRoom(false);
    };
    /*---------------------*/
    /* GUI RELATED METHODS */
    /*---------------------*/
    OpenViduAppearinComponent.prototype.onResize = function () {
        this.resizeStreamsManually();
    };
    OpenViduAppearinComponent.prototype.toggleMic = function () {
        this.openviduApi.micEnabled = !this.openviduApi.micEnabled;
    };
    OpenViduAppearinComponent.prototype.toggleCamera = function () {
        this.openviduApi.camEnabled = !this.openviduApi.camEnabled;
    };
    OpenViduAppearinComponent.prototype.toggleFullscreen = function () {
        if (this.bigScreenService.isFullscreen()) {
            this.bigScreenService.exit();
        }
        else {
            this.bigScreenService.request(this.mainElement.nativeElement);
        }
    };
    OpenViduAppearinComponent.prototype.toggleChat = function () {
        //this.sidenav.toggle();
        this.showChat = !this.showChat;
    };
    OpenViduAppearinComponent.prototype.sendMessage = function (text) {
        // Clean input
        this.messageInput.nativeElement.value = null;
        //this.renderer.setValue(this.messageInput, null);
        // Send to OpenVidu server
        this.openviduApi.sendMessage(text);
    };
    OpenViduAppearinComponent.prototype.leaveRoom = function (callLeaveRoom) {
        _super.prototype.leaveRoom.call(this, callLeaveRoom);
        // Display message
        this.setUserMessage(this._intl.youLeftTheRoomLabel);
    };
    /*------------------------*/
    /* HANDLE OPENVIDU EVENTS */
    /*------------------------*/
    OpenViduAppearinComponent.prototype.handleOnServerConnected = function () {
        _super.prototype.handleOnServerConnected.call(this);
        this.setUserMessage(this._intl.connectingToRoomLabel);
    };
    OpenViduAppearinComponent.prototype.handleOnErrorRoom = function () {
        _super.prototype.handleOnErrorRoom.call(this);
        this.setUserMessage(this._intl.errorRoom);
    };
    OpenViduAppearinComponent.prototype.handleOnCameraAccessChange = function (cameraEvent) {
        _super.prototype.handleOnCameraAccessChange.call(this, cameraEvent);
        if (cameraEvent.access) {
            this.welcome = false;
        }
    };
    OpenViduAppearinComponent.prototype.handleOnParticipantJoined = function (participantEvent) {
        _super.prototype.handleOnParticipantJoined.call(this, participantEvent);
        // Fix: manually resize panel
        this.resizeStreamsManually();
    };
    OpenViduAppearinComponent.prototype.handleOnParticipantLeft = function (participantEvent) {
        _super.prototype.handleOnParticipantLeft.call(this, participantEvent);
        // Fix: manually resize panel
        this.resizeStreamsManually();
    };
    OpenViduAppearinComponent.prototype.handleOnStreamAdded = function (streamEvent) {
        _super.prototype.handleOnStreamAdded.call(this, streamEvent);
        // Fix: manually resize panel
        this.resizeStreamsManually();
    };
    OpenViduAppearinComponent.prototype.handleOnStreamRemoved = function (streamEvent) {
        _super.prototype.handleOnStreamRemoved.call(this, streamEvent);
        // Fix: manually resize panel
        this.resizeStreamsManually();
    };
    OpenViduAppearinComponent.prototype.handleOnSourceAdded = function () {
        // Fix: manually resize panel
        this.resizeStreamsManually();
    };
    /*-----------------*/
    /* PRIVATE METHODS */
    /*-----------------*/
    OpenViduAppearinComponent.prototype.resizeStreamsManually = function () {
        var videoStreams = this.streamAppearin.map(function (stream) {
            return stream.videoStream;
        });
        var obj = this.auxResizeStreamsManually(this.panelVideo, videoStreams);
        if (!obj.error) {
            this.streamMaxWidth = obj.width + 'px';
            this.streamMaxHeight = obj.height + 'px';
        }
    };
    __decorate([
        core_1.Input(),
        __metadata("design:type", Array)
    ], OpenViduAppearinComponent.prototype, "toolbarOptions", void 0);
    __decorate([
        core_1.ViewChild('main'),
        __metadata("design:type", core_1.ElementRef)
    ], OpenViduAppearinComponent.prototype, "mainElement", void 0);
    __decorate([
        core_1.ViewChild('sidenav'),
        __metadata("design:type", material_1.MatSidenav)
    ], OpenViduAppearinComponent.prototype, "sidenav", void 0);
    __decorate([
        core_1.ViewChild('messageInput'),
        __metadata("design:type", core_1.ElementRef)
    ], OpenViduAppearinComponent.prototype, "messageInput", void 0);
    __decorate([
        core_1.ViewChild('panelVideo'),
        __metadata("design:type", core_1.ElementRef)
    ], OpenViduAppearinComponent.prototype, "panelVideo", void 0);
    __decorate([
        core_1.ViewChildren('streamAppearin'),
        __metadata("design:type", core_1.QueryList)
    ], OpenViduAppearinComponent.prototype, "streamAppearin", void 0);
    __decorate([
        core_1.HostListener('window:resize', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], OpenViduAppearinComponent.prototype, "onResize", null);
    OpenViduAppearinComponent = __decorate([
        core_1.Component({
            selector: 'openvidu-appearin',
            template: "\n   <openvidu-template\n   \t#openviduApi=\"openviduApi\"\n   \t[wsUrl]=\"wsUrl\"\n   \t[sessionId]=\"sessionId\"\n   \t[participantId]=\"participantId\"\n\t\n   \t(onUpdateMainSpeaker)=\"handleOnUpdateMainSpeaker($event)\"\n\t\n   \t(onServerConnected)=\"handleOnServerConnected($event)\"\n   \t(onErrorServer)=\"handleOnErrorServer($event)\"\n   \t(onRoomConnected)=\"handleOnRoomConnected($event)\"\n   \t(onErrorRoom)=\"handleOnErrorRoom($event)\"\n   \t(onCameraAccessChange)=\"handleOnCameraAccessChange($event)\"\n\t\n   \t(onParticipantJoined)=\"handleOnParticipantJoined($event)\"\n   \t(onParticipantLeft)=\"handleOnParticipantLeft($event)\"\n\t\n   \t(onStreamAdded)=\"handleOnStreamAdded($event)\"\n   \t(onStreamRemoved)=\"handleOnStreamRemoved($event)\"\n\t\n   \t(onErrorMedia)=\"handleOnErrorMedia()\"\n   \t(onLeaveRoom)=\"handleOnLeaveRoom()\"\n   \t(onNewMessage)=\"handleOnNewMessage($event)\"\n   \t(onCustomNotification)=\"handleOnCustomNotification($event)\">\n\n   \t<div #main class=\"openvidu-appearin\" [ngClass]=\"{'fullscreen': isFullscreen}\">\n\t\t\n   \t\t<!-- WELCOME MESSAGE -->\n   \t\t<div id=\"welcome\" *ngIf=\"welcome\">\n   \t\t\t<header class=\"room-connecting-content-header\">\n   \t\t\t\t<h1>{{_intl.youAreEnteringRoomLabel}}</h1>\n   \t\t\t</header>\n\n   \t\t\t<section class=\"body\" *ngIf=\"connectionUiState == ConnectionState.NOT_CONNECTED\">\n   \t\t\t\t<div class=\"ai-spinner\">\n   \t\t\t\t\t<svg version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xml:space=\"preserve\" id=\"Layer_1\" x=\"0px\" y=\"0px\" viewBox=\"0 0 38 38\">\n   \t\t\t\t\t\t<style type=\"text/css\">\n   \t\t\t\t\t\t\t.st0{fill:none;stroke:#D1D1D1;stroke-width:1;}\n   \t\t\t\t\t\t\t.st1{fill:none;stroke:#FF2E63;stroke-width:1;transform-origin:18px 18px;}\n   \t\t\t\t\t\t\t.st2{fill:none;stroke:#41CC8D;stroke-width:1;transform-origin:18px 18px;}\n   \t\t\t\t\t\t</style>\n   \t\t\t\t\t\t<g transform=\"translate(1 1)\">\n   \t\t\t\t\t\t\t<circle class=\"st0\" cx=\"18\" cy=\"18\" r=\"18\"></circle>\n   \t\t\t\t\t\t\t<path class=\"st1\" d=\"M36,18c0-9.9-8.1-18-18-18\"></path>\n   \t\t\t\t\t\t\t<path class=\"st2\" d=\"M18,36c9.9,0,18-8.1,18-18\"></path>\n   \t\t\t\t\t\t</g>\n   \t\t\t\t\t</svg>\n   \t\t\t\t</div>\n   \t\t\t\t<p class=\"text-state\">{{_intl.connectingLabel}}</p>\n   \t\t\t</section>\n\n   \t\t\t<section class=\"body\" *ngIf=\"connectionUiState == ConnectionState.CONNECTED_TO_SERVER\">\n   \t\t\t\t<div class=\"ai-spinner\">\n   \t\t\t\t\t<svg version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xml:space=\"preserve\" id=\"Layer_1\" x=\"0px\" y=\"0px\" viewBox=\"0 0 38 38\">\n   \t\t\t\t\t\t<style type=\"text/css\">\n   \t\t\t\t\t\t\t.st0{fill:none;stroke:#D1D1D1;stroke-width:1;}\n   \t\t\t\t\t\t\t.st1{fill:none;stroke:#FF2E63;stroke-width:1;transform-origin:18px 18px;}\n   \t\t\t\t\t\t\t.st2{fill:none;stroke:#41CC8D;stroke-width:1;transform-origin:18px 18px;}\n   \t\t\t\t\t\t</style>\n   \t\t\t\t\t\t<g transform=\"translate(1 1)\">\n   \t\t\t\t\t\t\t<circle class=\"st0\" cx=\"18\" cy=\"18\" r=\"18\"></circle>\n   \t\t\t\t\t\t\t<path class=\"st1\" d=\"M36,18c0-9.9-8.1-18-18-18\"></path>\n   \t\t\t\t\t\t\t<path class=\"st2\" d=\"M18,36c9.9,0,18-8.1,18-18\"></path>\n   \t\t\t\t\t\t</g>\n   \t\t\t\t\t</svg>\n   \t\t\t\t</div>\n   \t\t\t\t<p class=\"text-state\">{{_intl.joiningRoomLabel}}</p>\n   \t\t\t</section>\n\n   \t\t\t<section class=\"body\" *ngIf=\"connectionUiState == ConnectionState.CONNECTED_TO_ROOM || connectionUiState == ConnectionState.REQUESTING_CAMERA_ACCESS\">\n   \t\t\t\t<div class=\"ai-spinner\">\n   \t\t\t\t\t<svg version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xml:space=\"preserve\" id=\"Layer_1\" x=\"0px\" y=\"0px\" viewBox=\"0 0 38 38\">\n   \t\t\t\t\t\t<style type=\"text/css\">\n   \t\t\t\t\t\t\t.st0{fill:none;stroke:#D1D1D1;stroke-width:1;}\n   \t\t\t\t\t\t\t.st1{fill:none;stroke:#FF2E63;stroke-width:1;transform-origin:18px 18px;}\n   \t\t\t\t\t\t\t.st2{fill:none;stroke:#41CC8D;stroke-width:1;transform-origin:18px 18px;}\n   \t\t\t\t\t\t</style>\n   \t\t\t\t\t\t<g transform=\"translate(1 1)\">\n   \t\t\t\t\t\t\t<circle class=\"st0\" cx=\"18\" cy=\"18\" r=\"18\"></circle>\n   \t\t\t\t\t\t\t<path class=\"st1\" d=\"M36,18c0-9.9-8.1-18-18-18\"></path>\n   \t\t\t\t\t\t\t<path class=\"st2\" d=\"M18,36c9.9,0,18-8.1,18-18\"></path>\n   \t\t\t\t\t\t</g>\n   \t\t\t\t\t</svg>\n   \t\t\t\t</div>\n   \t\t\t\t<p class=\"text-state\">{{_intl.loadingCameraLabel}}</p>\n   \t\t\t</section>\n\n   \t\t\t<section class=\"body\" *ngIf=\"connectionUiState == ConnectionState.CAMERA_ACCESS_DENIED\">\n   \t\t\t\t<h2>{{_intl.browserBlockingLabel}}</h2>\n\n   \t\t\t\t<figure>\n   \t\t\t\t\t<img src=\"https://d1x2efl61akomv.cloudfront.net/images/blocked-camera-chrome.4c7620aa.png\" height=\"38\">\n   \t\t\t\t</figure>\n\n   \t\t\t\t<p class=\"text\">{{_intl.browserBlockingExplainOneLabel}}</p>\n   \t\t\t\t<p class=\"text\">{{_intl.browserBlockingExplainTwoLabel}}</p>\n   \t\t\t</section>\n   \t\t</div>\n\n   \t\t<!-- GOODBYE MESSAGE -->\n   \t\t<div class=\"feedback-wrapper\" *ngIf=\"!session && !welcome\">\n   \t\t\t<div class=\"thank-you-message\">\n   \t\t\t\t<p class=\"body\">You have left the room</p>\n   \t\t\t</div>\n   \t\t</div>\n\n   \t\t<div class=\"wrapper\" *ngIf=\"session && !welcome\">\n   \t\t\t<!-- TOOLBAR -->\n   \t\t\t<div class=\"toolbar\">\n   \t\t\t\t<mat-toolbar [color]=\"primary\">\n   \t\t\t\t\t<span>{{sessionId}}</span>\n   \t\t\t\t\t<span class=\"example-spacer\"></span>\n\t\t\t\t\t\n   \t\t\t\t\t<ng-container *ngFor=\"let option of toolbarOptions\">\n   \t\t\t\t\t\t<button mat-icon-button *ngIf=\"option.icon !== null\" (click)=\"option.onClick !== null && option.onClick()\" matTooltip=\"{{(option.label) ? option.label : ''}}\" matTooltipPosition=\"below\">\n   \t\t\t\t\t\t\t<mat-icon>{{option.icon}}</mat-icon>\n   \t\t\t\t\t\t</button>\n   \t\t\t\t\t</ng-container>\n\t\t\t\t\t\n   \t\t\t\t\t<!-- For fullscreen -->\n   \t\t\t\t\t<button mat-icon-button (click)=\"toggleFullscreen()\" *ngIf=\"!isFullscreen\" matTooltip=\"{{_intl.fullscreenLabel}}\" matTooltipPosition=\"below\">\n   \t\t\t\t\t\t<mat-icon>fullscreen</mat-icon>\n   \t\t\t\t\t</button>\n   \t\t\t\t\t<button mat-icon-button (click)=\"toggleFullscreen()\" *ngIf=\"isFullscreen\" matTooltip=\"{{_intl.exitFullscreenLabel}}\" matTooltipPosition=\"below\">\n   \t\t\t\t\t\t<mat-icon>clear</mat-icon>\n   \t\t\t\t\t</button>\n   \t\t\t\t</mat-toolbar>\n   \t\t\t</div>\n\n   \t\t\t<mat-sidenav-container>\n   \t\t\t\t<!-- CHAT BUTTON -->\n   \t\t\t\t<button mat-fab class=\"chat-button\" (click)=\"toggleChat()\" *ngIf=\"!showChat\">\n   \t\t\t\t\t<mat-icon>chat_bubble</mat-icon>\n   \t\t\t\t</button>\n\n   \t\t\t\t<div class=\"mat-sidenav-content\">\n   \t\t\t\t\t<!-- MAIN VIDEO -->\n   \t\t\t\t\t<div #panelVideo class=\"panel-videos\">\n   \t\t\t\t\t\t<div class=\"stream-flex\">\n   \t\t\t\t\t\t\t<stream-appearin #streamAppearin *ngFor=\"let s of streams\" class=\"stream\" [stream]=\"s\"\n   \t\t\t\t\t\t\t\t[style.max-width]=\"streamMaxWidth\"\n   \t\t\t\t\t\t\t\t[style.max-height]=\"streamMaxHeight\"\n   \t\t\t\t\t\t\t\t(onSourceAdded)=\"handleOnSourceAdded($event)\">\n   \t\t\t\t\t\t\t</stream-appearin>\n   \t\t\t\t\t\t</div>\n   \t\t\t\t\t\t<div class=\"chat\" *ngIf=\"showChat\">\n   \t\t\t\t\t\t\t<button class=\"close\" (click)=\"toggleChat()\">\n   \t\t\t\t\t\t\t\t<mat-icon>close</mat-icon>\n   \t\t\t\t\t\t\t</button>\n   \t\t\t\t\t\t\t<ul>\n   \t\t\t\t\t\t\t\t<li *ngFor=\"let message of chatMessages\" [ngClass]=\"{'me': message.username == participantId}\">\n   \t\t\t\t\t\t\t\t\t<p>\n   \t\t\t\t\t\t\t\t\t\t{{message.message}}\n   \t\t\t\t\t\t\t\t\t</p>\n   \t\t\t\t\t\t\t\t</li>\n   \t\t\t\t\t\t\t</ul>\n   \t\t\t\t\t\t\t<div class=\"message-input-container\" (submit)=\"sendMessage(messageInput.value);\" accept-charset=\"UTF-8\">\n   \t\t\t\t\t\t\t\t<form>\n   \t\t\t\t\t\t\t\t\t<mat-input-container color=\"primary\" floatPlaceholder=\"never\">\n   \t\t\t\t\t\t\t\t\t\t<input matInput #messageInput placeholder=\"{{_intl.sendAMessageLabel}}\">\n   \t\t\t\t\t\t\t\t\t</mat-input-container>\n   \t\t\t\t\t\t\t\t</form>\n   \t\t\t\t\t\t\t</div>\n   \t\t\t\t\t\t</div>\n   \t\t\t\t\t</div>\n\n   \t\t\t\t\t<!-- BUTTONS & OTHERS VIDEOS -->\n   \t\t\t\t\t<div class=\"bottom-bar\">\n   \t\t\t\t\t\t<div class=\"buttons\">\n   \t\t\t\t\t\t\t<button mat-button class=\"toggle-mic\" [ngClass]=\"{'disabled': !openviduApi.micEnabled}\" (click)=\"toggleMic()\">\n   \t\t\t\t\t\t\t\t<mat-icon>mic</mat-icon>\n   \t\t\t\t\t\t\t\t<mat-icon class=\"disabled-icon\">mic_off</mat-icon>\n   \t\t\t\t\t\t\t\t<div>{{_intl.muteLabel}}</div>\n   \t\t\t\t\t\t\t\t<div class=\"disabled-text\">{{_intl.unmuteLabel}}</div>\n   \t\t\t\t\t\t\t</button>\n   \t\t\t\t\t\t\t<button mat-fab class=\"close-session\" (click)=\"leaveRoom()\" title=\"Leave room\">\n   \t\t\t\t\t\t\t\t<mat-icon>close</mat-icon>\n   \t\t\t\t\t\t\t</button>\n   \t\t\t\t\t\t\t<button mat-button class=\"toggle-camera\" [ngClass]=\"{'disabled': !openviduApi.camEnabled}\" (click)=\"toggleCamera()\">\n   \t\t\t\t\t\t\t\t<mat-icon>videocam</mat-icon>\n   \t\t\t\t\t\t\t\t<mat-icon class=\"disabled-icon\">videocam_off</mat-icon>\n   \t\t\t\t\t\t\t\t<div>{{_intl.videoOffLabel}}</div>\n   \t\t\t\t\t\t\t\t<div class=\"disabled-text\">{{_intl.videoOnLabel}}</div>\n   \t\t\t\t\t\t\t</button>\n   \t\t\t\t\t\t</div>\n   \t\t\t\t\t</div>\n   \t\t\t\t</div>\n   \t\t\t</mat-sidenav-container>\n   \t\t</div>\n   \t</div>\n\t\n   </openvidu-template>\n\t",
            styles: ["\n   /**\n    * Applies styles for users in high contrast mode. Note that this only applies\n    * to Microsoft browsers. Chrome can be included by checking for the `html[hc]`\n    * attribute, however Chrome handles high contrast differently.\n    */\n   /* Theme for the ripple elements.*/\n   /* stylelint-disable material/no-prefixes */\n   /* stylelint-enable */\n   .mat-elevation-z0 {\n     -webkit-box-shadow: 0px 0px 0px 0px rgba(0, 0, 0, 0.2), 0px 0px 0px 0px rgba(0, 0, 0, 0.14), 0px 0px 0px 0px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 0px 0px 0px rgba(0, 0, 0, 0.2), 0px 0px 0px 0px rgba(0, 0, 0, 0.14), 0px 0px 0px 0px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z1 {\n     -webkit-box-shadow: 0px 2px 1px -1px rgba(0, 0, 0, 0.2), 0px 1px 1px 0px rgba(0, 0, 0, 0.14), 0px 1px 3px 0px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 2px 1px -1px rgba(0, 0, 0, 0.2), 0px 1px 1px 0px rgba(0, 0, 0, 0.14), 0px 1px 3px 0px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z2 {\n     -webkit-box-shadow: 0px 3px 1px -2px rgba(0, 0, 0, 0.2), 0px 2px 2px 0px rgba(0, 0, 0, 0.14), 0px 1px 5px 0px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 3px 1px -2px rgba(0, 0, 0, 0.2), 0px 2px 2px 0px rgba(0, 0, 0, 0.14), 0px 1px 5px 0px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z3 {\n     -webkit-box-shadow: 0px 3px 3px -2px rgba(0, 0, 0, 0.2), 0px 3px 4px 0px rgba(0, 0, 0, 0.14), 0px 1px 8px 0px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 3px 3px -2px rgba(0, 0, 0, 0.2), 0px 3px 4px 0px rgba(0, 0, 0, 0.14), 0px 1px 8px 0px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z4 {\n     -webkit-box-shadow: 0px 2px 4px -1px rgba(0, 0, 0, 0.2), 0px 4px 5px 0px rgba(0, 0, 0, 0.14), 0px 1px 10px 0px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 2px 4px -1px rgba(0, 0, 0, 0.2), 0px 4px 5px 0px rgba(0, 0, 0, 0.14), 0px 1px 10px 0px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z5 {\n     -webkit-box-shadow: 0px 3px 5px -1px rgba(0, 0, 0, 0.2), 0px 5px 8px 0px rgba(0, 0, 0, 0.14), 0px 1px 14px 0px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 3px 5px -1px rgba(0, 0, 0, 0.2), 0px 5px 8px 0px rgba(0, 0, 0, 0.14), 0px 1px 14px 0px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z6 {\n     -webkit-box-shadow: 0px 3px 5px -1px rgba(0, 0, 0, 0.2), 0px 6px 10px 0px rgba(0, 0, 0, 0.14), 0px 1px 18px 0px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 3px 5px -1px rgba(0, 0, 0, 0.2), 0px 6px 10px 0px rgba(0, 0, 0, 0.14), 0px 1px 18px 0px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z7 {\n     -webkit-box-shadow: 0px 4px 5px -2px rgba(0, 0, 0, 0.2), 0px 7px 10px 1px rgba(0, 0, 0, 0.14), 0px 2px 16px 1px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 4px 5px -2px rgba(0, 0, 0, 0.2), 0px 7px 10px 1px rgba(0, 0, 0, 0.14), 0px 2px 16px 1px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z8 {\n     -webkit-box-shadow: 0px 5px 5px -3px rgba(0, 0, 0, 0.2), 0px 8px 10px 1px rgba(0, 0, 0, 0.14), 0px 3px 14px 2px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 5px 5px -3px rgba(0, 0, 0, 0.2), 0px 8px 10px 1px rgba(0, 0, 0, 0.14), 0px 3px 14px 2px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z9 {\n     -webkit-box-shadow: 0px 5px 6px -3px rgba(0, 0, 0, 0.2), 0px 9px 12px 1px rgba(0, 0, 0, 0.14), 0px 3px 16px 2px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 5px 6px -3px rgba(0, 0, 0, 0.2), 0px 9px 12px 1px rgba(0, 0, 0, 0.14), 0px 3px 16px 2px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z10 {\n     -webkit-box-shadow: 0px 6px 6px -3px rgba(0, 0, 0, 0.2), 0px 10px 14px 1px rgba(0, 0, 0, 0.14), 0px 4px 18px 3px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 6px 6px -3px rgba(0, 0, 0, 0.2), 0px 10px 14px 1px rgba(0, 0, 0, 0.14), 0px 4px 18px 3px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z11 {\n     -webkit-box-shadow: 0px 6px 7px -4px rgba(0, 0, 0, 0.2), 0px 11px 15px 1px rgba(0, 0, 0, 0.14), 0px 4px 20px 3px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 6px 7px -4px rgba(0, 0, 0, 0.2), 0px 11px 15px 1px rgba(0, 0, 0, 0.14), 0px 4px 20px 3px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z12 {\n     -webkit-box-shadow: 0px 7px 8px -4px rgba(0, 0, 0, 0.2), 0px 12px 17px 2px rgba(0, 0, 0, 0.14), 0px 5px 22px 4px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 7px 8px -4px rgba(0, 0, 0, 0.2), 0px 12px 17px 2px rgba(0, 0, 0, 0.14), 0px 5px 22px 4px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z13 {\n     -webkit-box-shadow: 0px 7px 8px -4px rgba(0, 0, 0, 0.2), 0px 13px 19px 2px rgba(0, 0, 0, 0.14), 0px 5px 24px 4px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 7px 8px -4px rgba(0, 0, 0, 0.2), 0px 13px 19px 2px rgba(0, 0, 0, 0.14), 0px 5px 24px 4px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z14 {\n     -webkit-box-shadow: 0px 7px 9px -4px rgba(0, 0, 0, 0.2), 0px 14px 21px 2px rgba(0, 0, 0, 0.14), 0px 5px 26px 4px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 7px 9px -4px rgba(0, 0, 0, 0.2), 0px 14px 21px 2px rgba(0, 0, 0, 0.14), 0px 5px 26px 4px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z15 {\n     -webkit-box-shadow: 0px 8px 9px -5px rgba(0, 0, 0, 0.2), 0px 15px 22px 2px rgba(0, 0, 0, 0.14), 0px 6px 28px 5px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 8px 9px -5px rgba(0, 0, 0, 0.2), 0px 15px 22px 2px rgba(0, 0, 0, 0.14), 0px 6px 28px 5px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z16 {\n     -webkit-box-shadow: 0px 8px 10px -5px rgba(0, 0, 0, 0.2), 0px 16px 24px 2px rgba(0, 0, 0, 0.14), 0px 6px 30px 5px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 8px 10px -5px rgba(0, 0, 0, 0.2), 0px 16px 24px 2px rgba(0, 0, 0, 0.14), 0px 6px 30px 5px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z17 {\n     -webkit-box-shadow: 0px 8px 11px -5px rgba(0, 0, 0, 0.2), 0px 17px 26px 2px rgba(0, 0, 0, 0.14), 0px 6px 32px 5px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 8px 11px -5px rgba(0, 0, 0, 0.2), 0px 17px 26px 2px rgba(0, 0, 0, 0.14), 0px 6px 32px 5px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z18 {\n     -webkit-box-shadow: 0px 9px 11px -5px rgba(0, 0, 0, 0.2), 0px 18px 28px 2px rgba(0, 0, 0, 0.14), 0px 7px 34px 6px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 9px 11px -5px rgba(0, 0, 0, 0.2), 0px 18px 28px 2px rgba(0, 0, 0, 0.14), 0px 7px 34px 6px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z19 {\n     -webkit-box-shadow: 0px 9px 12px -6px rgba(0, 0, 0, 0.2), 0px 19px 29px 2px rgba(0, 0, 0, 0.14), 0px 7px 36px 6px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 9px 12px -6px rgba(0, 0, 0, 0.2), 0px 19px 29px 2px rgba(0, 0, 0, 0.14), 0px 7px 36px 6px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z20 {\n     -webkit-box-shadow: 0px 10px 13px -6px rgba(0, 0, 0, 0.2), 0px 20px 31px 3px rgba(0, 0, 0, 0.14), 0px 8px 38px 7px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 10px 13px -6px rgba(0, 0, 0, 0.2), 0px 20px 31px 3px rgba(0, 0, 0, 0.14), 0px 8px 38px 7px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z21 {\n     -webkit-box-shadow: 0px 10px 13px -6px rgba(0, 0, 0, 0.2), 0px 21px 33px 3px rgba(0, 0, 0, 0.14), 0px 8px 40px 7px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 10px 13px -6px rgba(0, 0, 0, 0.2), 0px 21px 33px 3px rgba(0, 0, 0, 0.14), 0px 8px 40px 7px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z22 {\n     -webkit-box-shadow: 0px 10px 14px -6px rgba(0, 0, 0, 0.2), 0px 22px 35px 3px rgba(0, 0, 0, 0.14), 0px 8px 42px 7px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 10px 14px -6px rgba(0, 0, 0, 0.2), 0px 22px 35px 3px rgba(0, 0, 0, 0.14), 0px 8px 42px 7px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z23 {\n     -webkit-box-shadow: 0px 11px 14px -7px rgba(0, 0, 0, 0.2), 0px 23px 36px 3px rgba(0, 0, 0, 0.14), 0px 9px 44px 8px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 11px 14px -7px rgba(0, 0, 0, 0.2), 0px 23px 36px 3px rgba(0, 0, 0, 0.14), 0px 9px 44px 8px rgba(0, 0, 0, 0.12); }\n\n   .mat-elevation-z24 {\n     -webkit-box-shadow: 0px 11px 15px -7px rgba(0, 0, 0, 0.2), 0px 24px 38px 3px rgba(0, 0, 0, 0.14), 0px 9px 46px 8px rgba(0, 0, 0, 0.12);\n             box-shadow: 0px 11px 15px -7px rgba(0, 0, 0, 0.2), 0px 24px 38px 3px rgba(0, 0, 0, 0.14), 0px 9px 46px 8px rgba(0, 0, 0, 0.12); }\n\n   .mat-h1, .mat-headline, .mat-typography h1 {\n     font: 400 24px/32px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0 0 16px; }\n\n   .mat-h2, .mat-title, .mat-typography h2 {\n     font: 500 20px/32px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0 0 16px; }\n\n   .mat-h3, .mat-subheading-2, .mat-typography h3 {\n     font: 400 16px/28px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0 0 16px; }\n\n   .mat-h4, .mat-subheading-1, .mat-typography h4 {\n     font: 400 15px/24px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0 0 16px; }\n\n   .mat-h5, .mat-typography h5 {\n     font: 400 11.62px/20px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0 0 12px; }\n\n   .mat-h6, .mat-typography h6 {\n     font: 400 9.38px/20px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0 0 12px; }\n\n   .mat-body-strong, .mat-body-2 {\n     font: 500 14px/24px Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-body, .mat-body-1, .mat-typography {\n     font: 400 14px/20px Roboto, \"Helvetica Neue\", sans-serif; }\n     .mat-body p, .mat-body-1 p, .mat-typography p {\n       margin: 0 0 12px; }\n\n   .mat-small, .mat-caption {\n     font: 400 12px/20px Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-display-4, .mat-typography .mat-display-4 {\n     font: 300 112px/112px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0 0 56px;\n     letter-spacing: -0.05em; }\n\n   .mat-display-3, .mat-typography .mat-display-3 {\n     font: 400 56px/56px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0 0 64px;\n     letter-spacing: -0.02em; }\n\n   .mat-display-2, .mat-typography .mat-display-2 {\n     font: 400 45px/48px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0 0 64px;\n     letter-spacing: -0.005em; }\n\n   .mat-display-1, .mat-typography .mat-display-1 {\n     font: 400 34px/40px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0 0 64px; }\n\n   .mat-button, .mat-raised-button, .mat-icon-button, .mat-stroked-button, .mat-flat-button,\n   .mat-fab, .mat-mini-fab {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 14px;\n     font-weight: 500; }\n\n   .mat-button-toggle {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-card {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-card-title {\n     font-size: 24px;\n     font-weight: 400; }\n\n   .mat-card-subtitle,\n   .mat-card-content,\n   .mat-card-header .mat-card-title {\n     font-size: 14px; }\n\n   .mat-checkbox {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-checkbox-layout .mat-checkbox-label {\n     line-height: 24px; }\n\n   .mat-chip {\n     font-size: 13px;\n     line-height: 18px; }\n     .mat-chip .mat-chip-remove.mat-icon {\n       font-size: 18px; }\n\n   .mat-table {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-header-cell {\n     font-size: 12px;\n     font-weight: 500; }\n\n   .mat-cell {\n     font-size: 14px; }\n\n   .mat-calendar {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-calendar-body {\n     font-size: 13px; }\n\n   .mat-calendar-body-label,\n   .mat-calendar-period-button {\n     font-size: 14px;\n     font-weight: 500; }\n\n   .mat-calendar-table-header th {\n     font-size: 11px;\n     font-weight: 400; }\n\n   .mat-dialog-title {\n     font: 500 20px/32px Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-expansion-panel-header {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 15px;\n     font-weight: 400; }\n\n   .mat-expansion-panel-content {\n     font: 400 14px/20px Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-form-field {\n     font-size: inherit;\n     font-weight: 400;\n     line-height: 1.125;\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-form-field-wrapper {\n     padding-bottom: 1.25em; }\n\n   .mat-form-field-prefix .mat-icon,\n   .mat-form-field-suffix .mat-icon {\n     font-size: 150%;\n     line-height: 1.125; }\n\n   .mat-form-field-prefix .mat-icon-button,\n   .mat-form-field-suffix .mat-icon-button {\n     height: 1.5em;\n     width: 1.5em; }\n     .mat-form-field-prefix .mat-icon-button .mat-icon,\n     .mat-form-field-suffix .mat-icon-button .mat-icon {\n       height: 1.125em;\n       line-height: 1.125; }\n\n   .mat-form-field-infix {\n     padding: 0.4375em 0;\n     border-top: 0.84375em solid transparent; }\n\n   .mat-form-field-can-float.mat-form-field-should-float .mat-form-field-label,\n   .mat-form-field-can-float .mat-input-server:focus + .mat-form-field-label-wrapper .mat-form-field-label {\n     -webkit-transform: translateY(-1.28125em) scale(0.75) perspective(100px) translateZ(0.001px);\n             transform: translateY(-1.28125em) scale(0.75) perspective(100px) translateZ(0.001px);\n     -ms-transform: translateY(-1.28125em) scale(0.75);\n     width: 133.33333%; }\n\n   .mat-form-field-can-float .mat-form-field-autofill-control:-webkit-autofill + .mat-form-field-label-wrapper\n   .mat-form-field-label {\n     -webkit-transform: translateY(-1.28125em) scale(0.75) perspective(100px) translateZ(0.00101px);\n             transform: translateY(-1.28125em) scale(0.75) perspective(100px) translateZ(0.00101px);\n     -ms-transform: translateY(-1.28124em) scale(0.75);\n     width: 133.33334%; }\n\n   .mat-form-field-can-float .mat-input-server[label]:not(:label-shown) + .mat-form-field-label-wrapper\n   .mat-form-field-label {\n     -webkit-transform: translateY(-1.28125em) scale(0.75) perspective(100px) translateZ(0.00102px);\n             transform: translateY(-1.28125em) scale(0.75) perspective(100px) translateZ(0.00102px);\n     -ms-transform: translateY(-1.28123em) scale(0.75);\n     width: 133.33335%; }\n\n   .mat-form-field-label-wrapper {\n     top: -0.84375em;\n     padding-top: 0.84375em; }\n\n   .mat-form-field-label {\n     top: 1.28125em; }\n\n   .mat-form-field-underline {\n     bottom: 1.25em; }\n\n   .mat-form-field-subscript-wrapper {\n     font-size: 75%;\n     margin-top: 0.54167em;\n     top: -webkit-calc(100% - 1.66667em);\n     top: calc(100% - 1.66667em); }\n\n   .mat-grid-tile-header,\n   .mat-grid-tile-footer {\n     font-size: 14px; }\n     .mat-grid-tile-header .mat-line,\n     .mat-grid-tile-footer .mat-line {\n       white-space: nowrap;\n       overflow: hidden;\n       -o-text-overflow: ellipsis;\n          text-overflow: ellipsis;\n       display: block;\n       -webkit-box-sizing: border-box;\n               box-sizing: border-box; }\n       .mat-grid-tile-header .mat-line:nth-child(n+2),\n       .mat-grid-tile-footer .mat-line:nth-child(n+2) {\n         font-size: 12px; }\n\n   input.mat-input-element {\n     margin-top: -0.0625em; }\n\n   .mat-menu-item {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 16px;\n     font-weight: 400; }\n\n   .mat-paginator,\n   .mat-paginator-page-size .mat-select-trigger {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 12px; }\n\n   .mat-radio-button {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-select {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-select-trigger {\n     height: 1.125em; }\n\n   .mat-slide-toggle-content {\n     font: 400 14px/20px Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-slider-thumb-label-text {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 12px;\n     font-weight: 500; }\n\n   .mat-stepper-vertical, .mat-stepper-horizontal {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-step-label {\n     font-size: 14px;\n     font-weight: 400; }\n\n   .mat-step-label-selected {\n     font-size: 14px;\n     font-weight: 500; }\n\n   .mat-tab-group {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-tab-label, .mat-tab-link {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 14px;\n     font-weight: 500; }\n\n   .mat-toolbar,\n   .mat-toolbar h1,\n   .mat-toolbar h2,\n   .mat-toolbar h3,\n   .mat-toolbar h4,\n   .mat-toolbar h5,\n   .mat-toolbar h6 {\n     font: 500 20px/32px Roboto, \"Helvetica Neue\", sans-serif;\n     margin: 0; }\n\n   .mat-tooltip {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 10px;\n     padding-top: 6px;\n     padding-bottom: 6px; }\n\n   .mat-tooltip-handset {\n     font-size: 14px;\n     padding-top: 9px;\n     padding-bottom: 9px; }\n\n   .mat-list-item {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-list-option {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-list .mat-list-item, .mat-nav-list .mat-list-item, .mat-selection-list .mat-list-item {\n     font-size: 16px; }\n     .mat-list .mat-list-item .mat-line, .mat-nav-list .mat-list-item .mat-line, .mat-selection-list .mat-list-item .mat-line {\n       white-space: nowrap;\n       overflow: hidden;\n       -o-text-overflow: ellipsis;\n          text-overflow: ellipsis;\n       display: block;\n       -webkit-box-sizing: border-box;\n               box-sizing: border-box; }\n       .mat-list .mat-list-item .mat-line:nth-child(n+2), .mat-nav-list .mat-list-item .mat-line:nth-child(n+2), .mat-selection-list .mat-list-item .mat-line:nth-child(n+2) {\n         font-size: 14px; }\n\n   .mat-list .mat-list-option, .mat-nav-list .mat-list-option, .mat-selection-list .mat-list-option {\n     font-size: 16px; }\n     .mat-list .mat-list-option .mat-line, .mat-nav-list .mat-list-option .mat-line, .mat-selection-list .mat-list-option .mat-line {\n       white-space: nowrap;\n       overflow: hidden;\n       -o-text-overflow: ellipsis;\n          text-overflow: ellipsis;\n       display: block;\n       -webkit-box-sizing: border-box;\n               box-sizing: border-box; }\n       .mat-list .mat-list-option .mat-line:nth-child(n+2), .mat-nav-list .mat-list-option .mat-line:nth-child(n+2), .mat-selection-list .mat-list-option .mat-line:nth-child(n+2) {\n         font-size: 14px; }\n\n   .mat-list .mat-subheader, .mat-nav-list .mat-subheader, .mat-selection-list .mat-subheader {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 14px;\n     font-weight: 500; }\n\n   .mat-list[dense] .mat-list-item, .mat-nav-list[dense] .mat-list-item, .mat-selection-list[dense] .mat-list-item {\n     font-size: 12px; }\n     .mat-list[dense] .mat-list-item .mat-line, .mat-nav-list[dense] .mat-list-item .mat-line, .mat-selection-list[dense] .mat-list-item .mat-line {\n       white-space: nowrap;\n       overflow: hidden;\n       -o-text-overflow: ellipsis;\n          text-overflow: ellipsis;\n       display: block;\n       -webkit-box-sizing: border-box;\n               box-sizing: border-box; }\n       .mat-list[dense] .mat-list-item .mat-line:nth-child(n+2), .mat-nav-list[dense] .mat-list-item .mat-line:nth-child(n+2), .mat-selection-list[dense] .mat-list-item .mat-line:nth-child(n+2) {\n         font-size: 12px; }\n\n   .mat-list[dense] .mat-list-option, .mat-nav-list[dense] .mat-list-option, .mat-selection-list[dense] .mat-list-option {\n     font-size: 12px; }\n     .mat-list[dense] .mat-list-option .mat-line, .mat-nav-list[dense] .mat-list-option .mat-line, .mat-selection-list[dense] .mat-list-option .mat-line {\n       white-space: nowrap;\n       overflow: hidden;\n       -o-text-overflow: ellipsis;\n          text-overflow: ellipsis;\n       display: block;\n       -webkit-box-sizing: border-box;\n               box-sizing: border-box; }\n       .mat-list[dense] .mat-list-option .mat-line:nth-child(n+2), .mat-nav-list[dense] .mat-list-option .mat-line:nth-child(n+2), .mat-selection-list[dense] .mat-list-option .mat-line:nth-child(n+2) {\n         font-size: 12px; }\n\n   .mat-list[dense] .mat-subheader, .mat-nav-list[dense] .mat-subheader, .mat-selection-list[dense] .mat-subheader {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 12px;\n     font-weight: 500; }\n\n   .mat-option {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 16px; }\n\n   .mat-optgroup-label {\n     font: 500 14px/24px Roboto, \"Helvetica Neue\", sans-serif; }\n\n   .mat-simple-snackbar {\n     font-family: Roboto, \"Helvetica Neue\", sans-serif;\n     font-size: 14px; }\n\n   .mat-simple-snackbar-action {\n     line-height: 1;\n     font-family: inherit;\n     font-size: inherit;\n     font-weight: 500; }\n\n   .mat-ripple {\n     overflow: hidden; }\n     @media screen and (-ms-high-contrast: active) {\n       .mat-ripple {\n         display: none; } }\n\n   .mat-ripple.mat-ripple-unbounded {\n     overflow: visible; }\n\n   .mat-ripple-element {\n     position: absolute;\n     -webkit-border-radius: 50%;\n             border-radius: 50%;\n     pointer-events: none;\n     -webkit-transition: opacity, -webkit-transform 0ms cubic-bezier(0, 0, 0.2, 1);\n     transition: opacity, -webkit-transform 0ms cubic-bezier(0, 0, 0.2, 1);\n     -o-transition: opacity, transform 0ms cubic-bezier(0, 0, 0.2, 1);\n     transition: opacity, transform 0ms cubic-bezier(0, 0, 0.2, 1);\n     transition: opacity, transform 0ms cubic-bezier(0, 0, 0.2, 1), -webkit-transform 0ms cubic-bezier(0, 0, 0.2, 1);\n     -webkit-transform: scale(0);\n         -ms-transform: scale(0);\n             transform: scale(0); }\n\n   .cdk-visually-hidden {\n     border: 0;\n     clip: rect(0 0 0 0);\n     height: 1px;\n     margin: -1px;\n     overflow: hidden;\n     padding: 0;\n     position: absolute;\n     width: 1px;\n     outline: 0;\n     -webkit-appearance: none;\n     -moz-appearance: none; }\n\n   .cdk-overlay-container, .cdk-global-overlay-wrapper {\n     pointer-events: none;\n     top: 0;\n     left: 0;\n     height: 100%;\n     width: 100%; }\n\n   .cdk-overlay-container {\n     position: fixed;\n     z-index: 1000; }\n\n   .cdk-global-overlay-wrapper {\n     display: -webkit-box;\n     display: -webkit-flex;\n     display: -ms-flexbox;\n     display: flex;\n     position: absolute;\n     z-index: 1000; }\n\n   .cdk-overlay-pane {\n     position: absolute;\n     pointer-events: auto;\n     -webkit-box-sizing: border-box;\n             box-sizing: border-box;\n     z-index: 1000; }\n\n   .cdk-overlay-backdrop {\n     position: absolute;\n     top: 0;\n     bottom: 0;\n     left: 0;\n     right: 0;\n     z-index: 1000;\n     pointer-events: auto;\n     -webkit-tap-highlight-color: transparent;\n     -webkit-transition: opacity 400ms cubic-bezier(0.25, 0.8, 0.25, 1);\n     -o-transition: opacity 400ms cubic-bezier(0.25, 0.8, 0.25, 1);\n     transition: opacity 400ms cubic-bezier(0.25, 0.8, 0.25, 1);\n     opacity: 0; }\n     .cdk-overlay-backdrop.cdk-overlay-backdrop-showing {\n       opacity: 1; }\n\n   .cdk-overlay-dark-backdrop {\n     background: rgba(0, 0, 0, 0.288); }\n\n   .cdk-overlay-transparent-backdrop, .cdk-overlay-transparent-backdrop.cdk-overlay-backdrop-showing {\n     opacity: 0; }\n\n   .cdk-global-scrollblock {\n     position: fixed;\n     width: 100%;\n     overflow-y: scroll; }\n\n   .mat-ripple-element {\n     background-color: rgba(0, 0, 0, 0.1); }\n\n   .mat-option {\n     color: rgba(0, 0, 0, 0.87); }\n     .mat-option:hover:not(.mat-option-disabled), .mat-option:focus:not(.mat-option-disabled) {\n       background: rgba(0, 0, 0, 0.04); }\n     .mat-primary .mat-option.mat-selected:not(.mat-option-disabled) {\n       color: #009688; }\n     .mat-accent .mat-option.mat-selected:not(.mat-option-disabled) {\n       color: #009688; }\n     .mat-warn .mat-option.mat-selected:not(.mat-option-disabled) {\n       color: #f44336; }\n     .mat-option.mat-selected:not(.mat-option-multiple):not(.mat-option-disabled) {\n       background: rgba(0, 0, 0, 0.04); }\n     .mat-option.mat-active {\n       background: rgba(0, 0, 0, 0.04);\n       color: rgba(0, 0, 0, 0.87); }\n     .mat-option.mat-option-disabled {\n       color: rgba(0, 0, 0, 0.38); }\n\n   .mat-optgroup-label {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-optgroup-disabled .mat-optgroup-label {\n     color: rgba(0, 0, 0, 0.38); }\n\n   .mat-pseudo-checkbox {\n     color: rgba(0, 0, 0, 0.54); }\n     .mat-pseudo-checkbox::after {\n       color: #fafafa; }\n\n   .mat-pseudo-checkbox-checked,\n   .mat-pseudo-checkbox-indeterminate,\n   .mat-accent .mat-pseudo-checkbox-checked,\n   .mat-accent .mat-pseudo-checkbox-indeterminate {\n     background: #009688; }\n\n   .mat-primary .mat-pseudo-checkbox-checked,\n   .mat-primary .mat-pseudo-checkbox-indeterminate {\n     background: #009688; }\n\n   .mat-warn .mat-pseudo-checkbox-checked,\n   .mat-warn .mat-pseudo-checkbox-indeterminate {\n     background: #f44336; }\n\n   .mat-pseudo-checkbox-checked.mat-pseudo-checkbox-disabled,\n   .mat-pseudo-checkbox-indeterminate.mat-pseudo-checkbox-disabled {\n     background: #b0b0b0; }\n\n   .mat-app-background {\n     background-color: #fafafa;\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-theme-loaded-marker {\n     display: none; }\n\n   .mat-autocomplete-panel {\n     background: white;\n     color: rgba(0, 0, 0, 0.87); }\n     .mat-autocomplete-panel .mat-option.mat-selected:not(.mat-active):not(:hover) {\n       background: white; }\n       .mat-autocomplete-panel .mat-option.mat-selected:not(.mat-active):not(:hover):not(.mat-option-disabled) {\n         color: rgba(0, 0, 0, 0.87); }\n\n   .mat-button, .mat-icon-button, .mat-stroked-button {\n     background: transparent; }\n     .mat-button.mat-primary .mat-button-focus-overlay, .mat-icon-button.mat-primary .mat-button-focus-overlay, .mat-stroked-button.mat-primary .mat-button-focus-overlay {\n       background-color: rgba(0, 150, 136, 0.12); }\n     .mat-button.mat-accent .mat-button-focus-overlay, .mat-icon-button.mat-accent .mat-button-focus-overlay, .mat-stroked-button.mat-accent .mat-button-focus-overlay {\n       background-color: rgba(0, 150, 136, 0.12); }\n     .mat-button.mat-warn .mat-button-focus-overlay, .mat-icon-button.mat-warn .mat-button-focus-overlay, .mat-stroked-button.mat-warn .mat-button-focus-overlay {\n       background-color: rgba(244, 67, 54, 0.12); }\n     .mat-button[disabled] .mat-button-focus-overlay, .mat-icon-button[disabled] .mat-button-focus-overlay, .mat-stroked-button[disabled] .mat-button-focus-overlay {\n       background-color: transparent; }\n     .mat-button.mat-primary, .mat-icon-button.mat-primary, .mat-stroked-button.mat-primary {\n       color: #009688; }\n     .mat-button.mat-accent, .mat-icon-button.mat-accent, .mat-stroked-button.mat-accent {\n       color: #009688; }\n     .mat-button.mat-warn, .mat-icon-button.mat-warn, .mat-stroked-button.mat-warn {\n       color: #f44336; }\n     .mat-button.mat-primary[disabled], .mat-button.mat-accent[disabled], .mat-button.mat-warn[disabled], .mat-button[disabled][disabled], .mat-icon-button.mat-primary[disabled], .mat-icon-button.mat-accent[disabled], .mat-icon-button.mat-warn[disabled], .mat-icon-button[disabled][disabled], .mat-stroked-button.mat-primary[disabled], .mat-stroked-button.mat-accent[disabled], .mat-stroked-button.mat-warn[disabled], .mat-stroked-button[disabled][disabled] {\n       color: rgba(0, 0, 0, 0.26); }\n\n   .mat-raised-button, .mat-fab, .mat-mini-fab {\n     color: rgba(0, 0, 0, 0.87);\n     background-color: white; }\n     .mat-raised-button.mat-primary, .mat-fab.mat-primary, .mat-mini-fab.mat-primary {\n       color: white; }\n     .mat-raised-button.mat-accent, .mat-fab.mat-accent, .mat-mini-fab.mat-accent {\n       color: white; }\n     .mat-raised-button.mat-warn, .mat-fab.mat-warn, .mat-mini-fab.mat-warn {\n       color: white; }\n     .mat-raised-button.mat-primary[disabled], .mat-raised-button.mat-accent[disabled], .mat-raised-button.mat-warn[disabled], .mat-raised-button[disabled][disabled], .mat-fab.mat-primary[disabled], .mat-fab.mat-accent[disabled], .mat-fab.mat-warn[disabled], .mat-fab[disabled][disabled], .mat-mini-fab.mat-primary[disabled], .mat-mini-fab.mat-accent[disabled], .mat-mini-fab.mat-warn[disabled], .mat-mini-fab[disabled][disabled] {\n       color: rgba(0, 0, 0, 0.26); }\n     .mat-raised-button.mat-primary, .mat-fab.mat-primary, .mat-mini-fab.mat-primary {\n       background-color: #009688; }\n     .mat-raised-button.mat-accent, .mat-fab.mat-accent, .mat-mini-fab.mat-accent {\n       background-color: #009688; }\n     .mat-raised-button.mat-warn, .mat-fab.mat-warn, .mat-mini-fab.mat-warn {\n       background-color: #f44336; }\n     .mat-raised-button.mat-primary[disabled], .mat-raised-button.mat-accent[disabled], .mat-raised-button.mat-warn[disabled], .mat-raised-button[disabled][disabled], .mat-fab.mat-primary[disabled], .mat-fab.mat-accent[disabled], .mat-fab.mat-warn[disabled], .mat-fab[disabled][disabled], .mat-mini-fab.mat-primary[disabled], .mat-mini-fab.mat-accent[disabled], .mat-mini-fab.mat-warn[disabled], .mat-mini-fab[disabled][disabled] {\n       background-color: rgba(0, 0, 0, 0.12); }\n     .mat-raised-button.mat-primary .mat-ripple-element, .mat-fab.mat-primary .mat-ripple-element, .mat-mini-fab.mat-primary .mat-ripple-element {\n       background-color: rgba(255, 255, 255, 0.2); }\n     .mat-raised-button.mat-accent .mat-ripple-element, .mat-fab.mat-accent .mat-ripple-element, .mat-mini-fab.mat-accent .mat-ripple-element {\n       background-color: rgba(255, 255, 255, 0.2); }\n     .mat-raised-button.mat-warn .mat-ripple-element, .mat-fab.mat-warn .mat-ripple-element, .mat-mini-fab.mat-warn .mat-ripple-element {\n       background-color: rgba(255, 255, 255, 0.2); }\n\n   .mat-button.mat-primary .mat-ripple-element {\n     background-color: rgba(0, 150, 136, 0.1); }\n\n   .mat-button.mat-accent .mat-ripple-element {\n     background-color: rgba(0, 150, 136, 0.1); }\n\n   .mat-button.mat-warn .mat-ripple-element {\n     background-color: rgba(244, 67, 54, 0.1); }\n\n   .mat-flat-button {\n     color: rgba(0, 0, 0, 0.87);\n     background-color: white; }\n     .mat-flat-button.mat-primary {\n       color: white; }\n     .mat-flat-button.mat-accent {\n       color: white; }\n     .mat-flat-button.mat-warn {\n       color: white; }\n     .mat-flat-button.mat-primary[disabled], .mat-flat-button.mat-accent[disabled], .mat-flat-button.mat-warn[disabled], .mat-flat-button[disabled][disabled] {\n       color: rgba(0, 0, 0, 0.26); }\n     .mat-flat-button.mat-primary {\n       background-color: #009688; }\n     .mat-flat-button.mat-accent {\n       background-color: #009688; }\n     .mat-flat-button.mat-warn {\n       background-color: #f44336; }\n     .mat-flat-button.mat-primary[disabled], .mat-flat-button.mat-accent[disabled], .mat-flat-button.mat-warn[disabled], .mat-flat-button[disabled][disabled] {\n       background-color: rgba(0, 0, 0, 0.12); }\n     .mat-flat-button.mat-primary .mat-ripple-element {\n       background-color: rgba(255, 255, 255, 0.2); }\n     .mat-flat-button.mat-accent .mat-ripple-element {\n       background-color: rgba(255, 255, 255, 0.2); }\n     .mat-flat-button.mat-warn .mat-ripple-element {\n       background-color: rgba(255, 255, 255, 0.2); }\n\n   .mat-icon-button.mat-primary .mat-ripple-element {\n     background-color: rgba(0, 150, 136, 0.2); }\n\n   .mat-icon-button.mat-accent .mat-ripple-element {\n     background-color: rgba(0, 150, 136, 0.2); }\n\n   .mat-icon-button.mat-warn .mat-ripple-element {\n     background-color: rgba(244, 67, 54, 0.2); }\n\n   .mat-button-toggle {\n     color: rgba(0, 0, 0, 0.38); }\n     .mat-button-toggle.cdk-focused .mat-button-toggle-focus-overlay {\n       background-color: rgba(0, 0, 0, 0.12); }\n\n   .mat-button-toggle-checked {\n     background-color: #e0e0e0;\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-button-toggle-disabled {\n     background-color: #eeeeee;\n     color: rgba(0, 0, 0, 0.26); }\n     .mat-button-toggle-disabled.mat-button-toggle-checked {\n       background-color: #bdbdbd; }\n\n   .mat-card {\n     background: white;\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-card-subtitle {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-checkbox-frame {\n     border-color: rgba(0, 0, 0, 0.54); }\n\n   .mat-checkbox-checkmark {\n     fill: #fafafa; }\n\n   .mat-checkbox-checkmark-path {\n     stroke: #fafafa !important; }\n\n   .mat-checkbox-mixedmark {\n     background-color: #fafafa; }\n\n   .mat-checkbox-indeterminate.mat-primary .mat-checkbox-background, .mat-checkbox-checked.mat-primary .mat-checkbox-background {\n     background-color: #009688; }\n\n   .mat-checkbox-indeterminate.mat-accent .mat-checkbox-background, .mat-checkbox-checked.mat-accent .mat-checkbox-background {\n     background-color: #009688; }\n\n   .mat-checkbox-indeterminate.mat-warn .mat-checkbox-background, .mat-checkbox-checked.mat-warn .mat-checkbox-background {\n     background-color: #f44336; }\n\n   .mat-checkbox-disabled.mat-checkbox-checked .mat-checkbox-background, .mat-checkbox-disabled.mat-checkbox-indeterminate .mat-checkbox-background {\n     background-color: #b0b0b0; }\n\n   .mat-checkbox-disabled:not(.mat-checkbox-checked) .mat-checkbox-frame {\n     border-color: #b0b0b0; }\n\n   .mat-checkbox-disabled .mat-checkbox-label {\n     color: #b0b0b0; }\n\n   .mat-checkbox:not(.mat-checkbox-disabled).mat-primary .mat-checkbox-ripple .mat-ripple-element {\n     background-color: rgba(0, 150, 136, 0.26); }\n\n   .mat-checkbox:not(.mat-checkbox-disabled).mat-accent .mat-checkbox-ripple .mat-ripple-element {\n     background-color: rgba(0, 150, 136, 0.26); }\n\n   .mat-checkbox:not(.mat-checkbox-disabled).mat-warn .mat-checkbox-ripple .mat-ripple-element {\n     background-color: rgba(244, 67, 54, 0.26); }\n\n   .mat-chip:not(.mat-basic-chip) {\n     background-color: #e0e0e0;\n     color: rgba(0, 0, 0, 0.87); }\n     .mat-chip:not(.mat-basic-chip) .mat-chip-remove {\n       color: rgba(0, 0, 0, 0.87);\n       opacity: 0.4; }\n     .mat-chip:not(.mat-basic-chip) .mat-chip-remove:hover {\n       opacity: 0.54; }\n\n   .mat-chip.mat-chip-selected.mat-primary {\n     background-color: #009688;\n     color: white; }\n     .mat-chip.mat-chip-selected.mat-primary .mat-chip-remove {\n       color: white;\n       opacity: 0.4; }\n     .mat-chip.mat-chip-selected.mat-primary .mat-chip-remove:hover {\n       opacity: 0.54; }\n\n   .mat-chip.mat-chip-selected.mat-warn {\n     background-color: #f44336;\n     color: white; }\n     .mat-chip.mat-chip-selected.mat-warn .mat-chip-remove {\n       color: white;\n       opacity: 0.4; }\n     .mat-chip.mat-chip-selected.mat-warn .mat-chip-remove:hover {\n       opacity: 0.54; }\n\n   .mat-chip.mat-chip-selected.mat-accent {\n     background-color: #009688;\n     color: white; }\n     .mat-chip.mat-chip-selected.mat-accent .mat-chip-remove {\n       color: white;\n       opacity: 0.4; }\n     .mat-chip.mat-chip-selected.mat-accent .mat-chip-remove:hover {\n       opacity: 0.54; }\n\n   .mat-table {\n     background: white; }\n\n   .mat-row, .mat-header-row {\n     border-bottom-color: rgba(0, 0, 0, 0.12); }\n\n   .mat-header-cell {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-cell {\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-datepicker-content {\n     background-color: white;\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-calendar-arrow {\n     border-top-color: rgba(0, 0, 0, 0.54); }\n\n   .mat-calendar-next-button,\n   .mat-calendar-previous-button {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-calendar-table-header {\n     color: rgba(0, 0, 0, 0.38); }\n\n   .mat-calendar-table-header-divider::after {\n     background: rgba(0, 0, 0, 0.12); }\n\n   .mat-calendar-body-label {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-calendar-body-cell-content {\n     color: rgba(0, 0, 0, 0.87);\n     border-color: transparent; }\n     .mat-calendar-body-disabled > .mat-calendar-body-cell-content:not(.mat-calendar-body-selected) {\n       color: rgba(0, 0, 0, 0.38); }\n\n   :not(.mat-calendar-body-disabled):hover > .mat-calendar-body-cell-content:not(.mat-calendar-body-selected),\n   .cdk-keyboard-focused .mat-calendar-body-active > .mat-calendar-body-cell-content:not(.mat-calendar-body-selected),\n   .cdk-program-focused .mat-calendar-body-active > .mat-calendar-body-cell-content:not(.mat-calendar-body-selected) {\n     background-color: rgba(0, 0, 0, 0.04); }\n\n   .mat-calendar-body-selected {\n     background-color: #009688;\n     color: white; }\n\n   .mat-calendar-body-disabled > .mat-calendar-body-selected {\n     background-color: rgba(0, 150, 136, 0.4); }\n\n   .mat-calendar-body-today:not(.mat-calendar-body-selected) {\n     border-color: rgba(0, 0, 0, 0.38); }\n\n   .mat-calendar-body-today.mat-calendar-body-selected {\n     -webkit-box-shadow: inset 0 0 0 1px white;\n             box-shadow: inset 0 0 0 1px white; }\n\n   .mat-calendar-body-disabled > .mat-calendar-body-today:not(.mat-calendar-body-selected) {\n     border-color: rgba(0, 0, 0, 0.18); }\n\n   .mat-datepicker-toggle-active {\n     color: #009688; }\n\n   .mat-dialog-container {\n     background: white;\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-divider {\n     border-top-color: rgba(0, 0, 0, 0.12); }\n\n   .mat-divider-vertical {\n     border-right-color: rgba(0, 0, 0, 0.12); }\n\n   .mat-expansion-panel {\n     background: white;\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-action-row {\n     border-top-color: rgba(0, 0, 0, 0.12); }\n\n   .mat-expansion-panel:not(.mat-expanded) .mat-expansion-panel-header:not([aria-disabled='true']).cdk-keyboard-focused, .mat-expansion-panel:not(.mat-expanded) .mat-expansion-panel-header:not([aria-disabled='true']).cdk-program-focused, .mat-expansion-panel:not(.mat-expanded) .mat-expansion-panel-header:not([aria-disabled='true']):hover {\n     background: rgba(0, 0, 0, 0.04); }\n\n   .mat-expansion-panel-header-title {\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-expansion-panel-header-description,\n   .mat-expansion-indicator::after {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-expansion-panel-header[aria-disabled='true'] {\n     color: rgba(0, 0, 0, 0.26); }\n     .mat-expansion-panel-header[aria-disabled='true'] .mat-expansion-panel-header-title,\n     .mat-expansion-panel-header[aria-disabled='true'] .mat-expansion-panel-header-description {\n       color: inherit; }\n\n   .mat-form-field-label {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-hint {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-focused .mat-form-field-label {\n     color: #009688; }\n     .mat-focused .mat-form-field-label.mat-accent {\n       color: #009688; }\n     .mat-focused .mat-form-field-label.mat-warn {\n       color: #f44336; }\n\n   .mat-focused .mat-form-field-required-marker {\n     color: #009688; }\n\n   .mat-form-field-underline {\n     background-color: rgba(0, 0, 0, 0.42); }\n\n   .mat-form-field-disabled .mat-form-field-underline {\n     background-image: -webkit-gradient(linear, left top, right top, from(rgba(0, 0, 0, 0.42)), color-stop(33%, rgba(0, 0, 0, 0.42)), color-stop(0%, transparent));\n     background-image: -webkit-linear-gradient(left, rgba(0, 0, 0, 0.42) 0%, rgba(0, 0, 0, 0.42) 33%, transparent 0%);\n     background-image: -o-linear-gradient(left, rgba(0, 0, 0, 0.42) 0%, rgba(0, 0, 0, 0.42) 33%, transparent 0%);\n     background-image: linear-gradient(to right, rgba(0, 0, 0, 0.42) 0%, rgba(0, 0, 0, 0.42) 33%, transparent 0%);\n     -webkit-background-size: 4px 1px;\n             background-size: 4px 1px;\n     background-repeat: repeat-x; }\n\n   .mat-form-field-ripple {\n     background-color: #009688; }\n     .mat-form-field-ripple.mat-accent {\n       background-color: #009688; }\n     .mat-form-field-ripple.mat-warn {\n       background-color: #f44336; }\n\n   .mat-form-field-invalid .mat-form-field-label {\n     color: #f44336; }\n     .mat-form-field-invalid .mat-form-field-label.mat-accent,\n     .mat-form-field-invalid .mat-form-field-label .mat-form-field-required-marker {\n       color: #f44336; }\n\n   .mat-form-field-invalid .mat-form-field-ripple {\n     background-color: #f44336; }\n\n   .mat-error {\n     color: #f44336; }\n\n   .mat-icon.mat-primary {\n     color: #009688; }\n\n   .mat-icon.mat-accent {\n     color: #009688; }\n\n   .mat-icon.mat-warn {\n     color: #f44336; }\n\n   .mat-input-element:disabled {\n     color: rgba(0, 0, 0, 0.38); }\n\n   .mat-input-element {\n     caret-color: #009688; }\n     .mat-input-element::-webkit-input-placeholder {\n       color: rgba(0, 0, 0, 0.42); }\n     .mat-input-element::-moz-placeholder {\n       color: rgba(0, 0, 0, 0.42); }\n     .mat-input-element:-ms-input-placeholder {\n       color: rgba(0, 0, 0, 0.42); }\n     .mat-input-element::-ms-input-placeholder {\n       color: rgba(0, 0, 0, 0.42); }\n     .mat-input-element::placeholder {\n       color: rgba(0, 0, 0, 0.42); }\n     .mat-input-element::-moz-placeholder {\n       color: rgba(0, 0, 0, 0.42); }\n     .mat-input-element::-webkit-input-placeholder {\n       color: rgba(0, 0, 0, 0.42); }\n     .mat-input-element:-ms-input-placeholder {\n       color: rgba(0, 0, 0, 0.42); }\n\n   .mat-accent .mat-input-element {\n     caret-color: #009688; }\n\n   .mat-warn .mat-input-element,\n   .mat-form-field-invalid .mat-input-element {\n     caret-color: #f44336; }\n\n   .mat-list .mat-list-item, .mat-nav-list .mat-list-item, .mat-selection-list .mat-list-item {\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-list .mat-list-option, .mat-nav-list .mat-list-option, .mat-selection-list .mat-list-option {\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-list .mat-subheader, .mat-nav-list .mat-subheader, .mat-selection-list .mat-subheader {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-list-item-disabled {\n     background-color: #eeeeee; }\n\n   .mat-nav-list .mat-list-item {\n     outline: none; }\n     .mat-nav-list .mat-list-item:hover, .mat-nav-list .mat-list-item.mat-list-item-focus {\n       background: rgba(0, 0, 0, 0.04); }\n\n   .mat-list-option {\n     outline: none; }\n     .mat-list-option:hover, .mat-list-option.mat-list-item-focus {\n       background: rgba(0, 0, 0, 0.04); }\n\n   .mat-menu-panel {\n     background: white; }\n\n   .mat-menu-item {\n     background: transparent;\n     color: rgba(0, 0, 0, 0.87); }\n     .mat-menu-item[disabled] {\n       color: rgba(0, 0, 0, 0.38); }\n\n   .mat-menu-item .mat-icon:not([color]),\n   .mat-menu-item-submenu-trigger::after {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-menu-item:hover:not([disabled]),\n   .mat-menu-item.cdk-program-focused:not([disabled]),\n   .mat-menu-item.cdk-keyboard-focused:not([disabled]),\n   .mat-menu-item-highlighted:not([disabled]) {\n     background: rgba(0, 0, 0, 0.04); }\n\n   .mat-paginator {\n     background: white; }\n\n   .mat-paginator,\n   .mat-paginator-page-size .mat-select-trigger {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-paginator-decrement,\n   .mat-paginator-increment {\n     border-top: 2px solid rgba(0, 0, 0, 0.54);\n     border-right: 2px solid rgba(0, 0, 0, 0.54); }\n\n   .mat-paginator-first,\n   .mat-paginator-last {\n     border-top: 2px solid rgba(0, 0, 0, 0.54); }\n\n   .mat-icon-button[disabled] .mat-paginator-decrement,\n   .mat-icon-button[disabled] .mat-paginator-increment,\n   .mat-icon-button[disabled] .mat-paginator-first,\n   .mat-icon-button[disabled] .mat-paginator-last {\n     border-color: rgba(0, 0, 0, 0.38); }\n\n   .mat-progress-bar-background {\n     fill: #b3e0db; }\n\n   .mat-progress-bar-buffer {\n     background-color: #b3e0db; }\n\n   .mat-progress-bar-fill::after {\n     background-color: #009688; }\n\n   .mat-progress-bar.mat-accent .mat-progress-bar-background {\n     fill: #b3e0db; }\n\n   .mat-progress-bar.mat-accent .mat-progress-bar-buffer {\n     background-color: #b3e0db; }\n\n   .mat-progress-bar.mat-accent .mat-progress-bar-fill::after {\n     background-color: #009688; }\n\n   .mat-progress-bar.mat-warn .mat-progress-bar-background {\n     fill: #ffcdd2; }\n\n   .mat-progress-bar.mat-warn .mat-progress-bar-buffer {\n     background-color: #ffcdd2; }\n\n   .mat-progress-bar.mat-warn .mat-progress-bar-fill::after {\n     background-color: #f44336; }\n\n   .mat-progress-spinner circle, .mat-spinner circle {\n     stroke: #009688; }\n\n   .mat-progress-spinner.mat-accent circle, .mat-spinner.mat-accent circle {\n     stroke: #009688; }\n\n   .mat-progress-spinner.mat-warn circle, .mat-spinner.mat-warn circle {\n     stroke: #f44336; }\n\n   .mat-radio-outer-circle {\n     border-color: rgba(0, 0, 0, 0.54); }\n\n   .mat-radio-disabled .mat-radio-outer-circle {\n     border-color: rgba(0, 0, 0, 0.38); }\n\n   .mat-radio-disabled .mat-radio-ripple .mat-ripple-element, .mat-radio-disabled .mat-radio-inner-circle {\n     background-color: rgba(0, 0, 0, 0.38); }\n\n   .mat-radio-disabled .mat-radio-label-content {\n     color: rgba(0, 0, 0, 0.38); }\n\n   .mat-radio-button.mat-primary.mat-radio-checked .mat-radio-outer-circle {\n     border-color: #009688; }\n\n   .mat-radio-button.mat-primary .mat-radio-inner-circle {\n     background-color: #009688; }\n\n   .mat-radio-button.mat-primary .mat-radio-ripple .mat-ripple-element {\n     background-color: rgba(0, 150, 136, 0.26); }\n\n   .mat-radio-button.mat-accent.mat-radio-checked .mat-radio-outer-circle {\n     border-color: #009688; }\n\n   .mat-radio-button.mat-accent .mat-radio-inner-circle {\n     background-color: #009688; }\n\n   .mat-radio-button.mat-accent .mat-radio-ripple .mat-ripple-element {\n     background-color: rgba(0, 150, 136, 0.26); }\n\n   .mat-radio-button.mat-warn.mat-radio-checked .mat-radio-outer-circle {\n     border-color: #f44336; }\n\n   .mat-radio-button.mat-warn .mat-radio-inner-circle {\n     background-color: #f44336; }\n\n   .mat-radio-button.mat-warn .mat-radio-ripple .mat-ripple-element {\n     background-color: rgba(244, 67, 54, 0.26); }\n\n   .mat-select-content, .mat-select-panel-done-animating {\n     background: white; }\n\n   .mat-select-value {\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-select-placeholder {\n     color: rgba(0, 0, 0, 0.42); }\n\n   .mat-select-disabled .mat-select-value {\n     color: rgba(0, 0, 0, 0.38); }\n\n   .mat-select-arrow {\n     color: rgba(0, 0, 0, 0.54); }\n\n   .mat-select-panel .mat-option.mat-selected:not(.mat-option-multiple) {\n     background: rgba(0, 0, 0, 0.12); }\n\n   .mat-form-field.mat-focused.mat-primary .mat-select-arrow {\n     color: #009688; }\n\n   .mat-form-field.mat-focused.mat-accent .mat-select-arrow {\n     color: #009688; }\n\n   .mat-form-field.mat-focused.mat-warn .mat-select-arrow {\n     color: #f44336; }\n\n   .mat-form-field .mat-select.mat-select-invalid .mat-select-arrow {\n     color: #f44336; }\n\n   .mat-form-field .mat-select.mat-select-disabled .mat-select-arrow {\n     color: rgba(0, 0, 0, 0.38); }\n\n   .mat-drawer-container {\n     background-color: #fafafa;\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-drawer {\n     background-color: white;\n     color: rgba(0, 0, 0, 0.87); }\n     .mat-drawer.mat-drawer-push {\n       background-color: white; }\n\n   .mat-drawer-backdrop.mat-drawer-shown {\n     background-color: rgba(0, 0, 0, 0.6); }\n\n   .mat-slide-toggle.mat-checked:not(.mat-disabled) .mat-slide-toggle-thumb {\n     background-color: #009688; }\n\n   .mat-slide-toggle.mat-checked:not(.mat-disabled) .mat-slide-toggle-bar {\n     background-color: rgba(0, 150, 136, 0.5); }\n\n   .mat-slide-toggle:not(.mat-checked) .mat-ripple-element {\n     background-color: rgba(0, 0, 0, 0.06); }\n\n   .mat-slide-toggle .mat-ripple-element {\n     background-color: rgba(0, 150, 136, 0.12); }\n\n   .mat-slide-toggle.mat-primary.mat-checked:not(.mat-disabled) .mat-slide-toggle-thumb {\n     background-color: #009688; }\n\n   .mat-slide-toggle.mat-primary.mat-checked:not(.mat-disabled) .mat-slide-toggle-bar {\n     background-color: rgba(0, 150, 136, 0.5); }\n\n   .mat-slide-toggle.mat-primary:not(.mat-checked) .mat-ripple-element {\n     background-color: rgba(0, 0, 0, 0.06); }\n\n   .mat-slide-toggle.mat-primary .mat-ripple-element {\n     background-color: rgba(0, 150, 136, 0.12); }\n\n   .mat-slide-toggle.mat-warn.mat-checked:not(.mat-disabled) .mat-slide-toggle-thumb {\n     background-color: #f44336; }\n\n   .mat-slide-toggle.mat-warn.mat-checked:not(.mat-disabled) .mat-slide-toggle-bar {\n     background-color: rgba(244, 67, 54, 0.5); }\n\n   .mat-slide-toggle.mat-warn:not(.mat-checked) .mat-ripple-element {\n     background-color: rgba(0, 0, 0, 0.06); }\n\n   .mat-slide-toggle.mat-warn .mat-ripple-element {\n     background-color: rgba(244, 67, 54, 0.12); }\n\n   .mat-disabled .mat-slide-toggle-thumb {\n     background-color: #bdbdbd; }\n\n   .mat-disabled .mat-slide-toggle-bar {\n     background-color: rgba(0, 0, 0, 0.1); }\n\n   .mat-slide-toggle-thumb {\n     background-color: #fafafa; }\n\n   .mat-slide-toggle-bar {\n     background-color: rgba(0, 0, 0, 0.38); }\n\n   .mat-slider-track-background {\n     background-color: rgba(0, 0, 0, 0.26); }\n\n   .mat-primary .mat-slider-track-fill,\n   .mat-primary .mat-slider-thumb,\n   .mat-primary .mat-slider-thumb-label {\n     background-color: #009688; }\n\n   .mat-primary .mat-slider-thumb-label-text {\n     color: white; }\n\n   .mat-accent .mat-slider-track-fill,\n   .mat-accent .mat-slider-thumb,\n   .mat-accent .mat-slider-thumb-label {\n     background-color: #009688; }\n\n   .mat-accent .mat-slider-thumb-label-text {\n     color: white; }\n\n   .mat-warn .mat-slider-track-fill,\n   .mat-warn .mat-slider-thumb,\n   .mat-warn .mat-slider-thumb-label {\n     background-color: #f44336; }\n\n   .mat-warn .mat-slider-thumb-label-text {\n     color: white; }\n\n   .mat-slider-focus-ring {\n     background-color: rgba(0, 150, 136, 0.2); }\n\n   .mat-slider:hover .mat-slider-track-background,\n   .cdk-focused .mat-slider-track-background {\n     background-color: rgba(0, 0, 0, 0.38); }\n\n   .mat-slider-disabled .mat-slider-track-background,\n   .mat-slider-disabled .mat-slider-track-fill,\n   .mat-slider-disabled .mat-slider-thumb {\n     background-color: rgba(0, 0, 0, 0.26); }\n\n   .mat-slider-disabled:hover .mat-slider-track-background {\n     background-color: rgba(0, 0, 0, 0.26); }\n\n   .mat-slider-min-value .mat-slider-focus-ring {\n     background-color: rgba(0, 0, 0, 0.12); }\n\n   .mat-slider-min-value.mat-slider-thumb-label-showing .mat-slider-thumb,\n   .mat-slider-min-value.mat-slider-thumb-label-showing .mat-slider-thumb-label {\n     background-color: rgba(0, 0, 0, 0.87); }\n\n   .mat-slider-min-value.mat-slider-thumb-label-showing.cdk-focused .mat-slider-thumb,\n   .mat-slider-min-value.mat-slider-thumb-label-showing.cdk-focused .mat-slider-thumb-label {\n     background-color: rgba(0, 0, 0, 0.26); }\n\n   .mat-slider-min-value:not(.mat-slider-thumb-label-showing) .mat-slider-thumb {\n     border-color: rgba(0, 0, 0, 0.26);\n     background-color: transparent; }\n\n   .mat-slider-min-value:not(.mat-slider-thumb-label-showing):hover .mat-slider-thumb, .mat-slider-min-value:not(.mat-slider-thumb-label-showing).cdk-focused .mat-slider-thumb {\n     border-color: rgba(0, 0, 0, 0.38); }\n\n   .mat-slider-min-value:not(.mat-slider-thumb-label-showing):hover.mat-slider-disabled .mat-slider-thumb, .mat-slider-min-value:not(.mat-slider-thumb-label-showing).cdk-focused.mat-slider-disabled .mat-slider-thumb {\n     border-color: rgba(0, 0, 0, 0.26); }\n\n   .mat-slider-has-ticks .mat-slider-wrapper::after {\n     border-color: rgba(0, 0, 0, 0.7); }\n\n   .mat-slider-horizontal .mat-slider-ticks {\n     background-image: -webkit-repeating-linear-gradient(left, rgba(0, 0, 0, 0.7), rgba(0, 0, 0, 0.7) 2px, transparent 0, transparent);\n     background-image: -o-repeating-linear-gradient(left, rgba(0, 0, 0, 0.7), rgba(0, 0, 0, 0.7) 2px, transparent 0, transparent);\n     background-image: repeating-linear-gradient(to right, rgba(0, 0, 0, 0.7), rgba(0, 0, 0, 0.7) 2px, transparent 0, transparent);\n     background-image: -moz-repeating-linear-gradient(0.0001deg, rgba(0, 0, 0, 0.7), rgba(0, 0, 0, 0.7) 2px, transparent 0, transparent); }\n\n   .mat-slider-vertical .mat-slider-ticks {\n     background-image: -webkit-repeating-linear-gradient(top, rgba(0, 0, 0, 0.7), rgba(0, 0, 0, 0.7) 2px, transparent 0, transparent);\n     background-image: -o-repeating-linear-gradient(top, rgba(0, 0, 0, 0.7), rgba(0, 0, 0, 0.7) 2px, transparent 0, transparent);\n     background-image: repeating-linear-gradient(to bottom, rgba(0, 0, 0, 0.7), rgba(0, 0, 0, 0.7) 2px, transparent 0, transparent); }\n\n   .mat-step-header.cdk-keyboard-focused, .mat-step-header.cdk-program-focused, .mat-step-header:hover {\n     background-color: rgba(0, 0, 0, 0.04); }\n\n   .mat-step-header .mat-step-label,\n   .mat-step-header .mat-step-optional {\n     color: rgba(0, 0, 0, 0.38); }\n\n   .mat-step-header .mat-step-icon {\n     background-color: #009688;\n     color: white; }\n\n   .mat-step-header .mat-step-icon-not-touched {\n     background-color: rgba(0, 0, 0, 0.38);\n     color: white; }\n\n   .mat-step-header .mat-step-label.mat-step-label-active {\n     color: rgba(0, 0, 0, 0.87); }\n\n   .mat-stepper-horizontal, .mat-stepper-vertical {\n     background-color: white; }\n\n   .mat-stepper-vertical-line::before {\n     border-left-color: rgba(0, 0, 0, 0.12); }\n\n   .mat-stepper-horizontal-line {\n     border-top-color: rgba(0, 0, 0, 0.12); }\n\n   .mat-tab-nav-bar,\n   .mat-tab-header {\n     border-bottom: 1px solid rgba(0, 0, 0, 0.12); }\n\n   .mat-tab-group-inverted-header .mat-tab-nav-bar,\n   .mat-tab-group-inverted-header .mat-tab-header {\n     border-top: 1px solid rgba(0, 0, 0, 0.12);\n     border-bottom: none; }\n\n   .mat-tab-label, .mat-tab-link {\n     color: rgba(0, 0, 0, 0.87); }\n     .mat-tab-label.mat-tab-disabled, .mat-tab-link.mat-tab-disabled {\n       color: rgba(0, 0, 0, 0.38); }\n\n   .mat-tab-header-pagination-chevron {\n     border-color: rgba(0, 0, 0, 0.87); }\n\n   .mat-tab-header-pagination-disabled .mat-tab-header-pagination-chevron {\n     border-color: rgba(0, 0, 0, 0.38); }\n\n   .mat-tab-group[class*='mat-background-'] .mat-tab-header,\n   .mat-tab-nav-bar[class*='mat-background-'] {\n     border-bottom: none;\n     border-top: none; }\n\n   .mat-tab-group.mat-primary .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-group.mat-primary .mat-tab-link:not(.mat-tab-disabled):focus, .mat-tab-nav-bar.mat-primary .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-nav-bar.mat-primary .mat-tab-link:not(.mat-tab-disabled):focus {\n     background-color: rgba(179, 224, 219, 0.3); }\n\n   .mat-tab-group.mat-primary .mat-ink-bar, .mat-tab-nav-bar.mat-primary .mat-ink-bar {\n     background-color: #009688; }\n\n   .mat-tab-group.mat-primary.mat-background-primary .mat-ink-bar, .mat-tab-nav-bar.mat-primary.mat-background-primary .mat-ink-bar {\n     background-color: white; }\n\n   .mat-tab-group.mat-accent .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-group.mat-accent .mat-tab-link:not(.mat-tab-disabled):focus, .mat-tab-nav-bar.mat-accent .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-nav-bar.mat-accent .mat-tab-link:not(.mat-tab-disabled):focus {\n     background-color: rgba(179, 224, 219, 0.3); }\n\n   .mat-tab-group.mat-accent .mat-ink-bar, .mat-tab-nav-bar.mat-accent .mat-ink-bar {\n     background-color: #009688; }\n\n   .mat-tab-group.mat-accent.mat-background-accent .mat-ink-bar, .mat-tab-nav-bar.mat-accent.mat-background-accent .mat-ink-bar {\n     background-color: white; }\n\n   .mat-tab-group.mat-warn .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-group.mat-warn .mat-tab-link:not(.mat-tab-disabled):focus, .mat-tab-nav-bar.mat-warn .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-nav-bar.mat-warn .mat-tab-link:not(.mat-tab-disabled):focus {\n     background-color: rgba(255, 205, 210, 0.3); }\n\n   .mat-tab-group.mat-warn .mat-ink-bar, .mat-tab-nav-bar.mat-warn .mat-ink-bar {\n     background-color: #f44336; }\n\n   .mat-tab-group.mat-warn.mat-background-warn .mat-ink-bar, .mat-tab-nav-bar.mat-warn.mat-background-warn .mat-ink-bar {\n     background-color: white; }\n\n   .mat-tab-group.mat-background-primary .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-group.mat-background-primary .mat-tab-link:not(.mat-tab-disabled):focus, .mat-tab-nav-bar.mat-background-primary .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-nav-bar.mat-background-primary .mat-tab-link:not(.mat-tab-disabled):focus {\n     background-color: rgba(179, 224, 219, 0.3); }\n\n   .mat-tab-group.mat-background-primary .mat-tab-header, .mat-tab-group.mat-background-primary .mat-tab-links, .mat-tab-nav-bar.mat-background-primary .mat-tab-header, .mat-tab-nav-bar.mat-background-primary .mat-tab-links {\n     background-color: #009688; }\n\n   .mat-tab-group.mat-background-primary .mat-tab-label, .mat-tab-group.mat-background-primary .mat-tab-link, .mat-tab-nav-bar.mat-background-primary .mat-tab-label, .mat-tab-nav-bar.mat-background-primary .mat-tab-link {\n     color: white; }\n     .mat-tab-group.mat-background-primary .mat-tab-label.mat-tab-disabled, .mat-tab-group.mat-background-primary .mat-tab-link.mat-tab-disabled, .mat-tab-nav-bar.mat-background-primary .mat-tab-label.mat-tab-disabled, .mat-tab-nav-bar.mat-background-primary .mat-tab-link.mat-tab-disabled {\n       color: rgba(255, 255, 255, 0.4); }\n\n   .mat-tab-group.mat-background-primary .mat-tab-header-pagination-chevron, .mat-tab-nav-bar.mat-background-primary .mat-tab-header-pagination-chevron {\n     border-color: white; }\n\n   .mat-tab-group.mat-background-primary .mat-tab-header-pagination-disabled .mat-tab-header-pagination-chevron, .mat-tab-nav-bar.mat-background-primary .mat-tab-header-pagination-disabled .mat-tab-header-pagination-chevron {\n     border-color: rgba(255, 255, 255, 0.4); }\n\n   .mat-tab-group.mat-background-primary .mat-ripple-element, .mat-tab-nav-bar.mat-background-primary .mat-ripple-element {\n     background-color: rgba(255, 255, 255, 0.12); }\n\n   .mat-tab-group.mat-background-accent .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-group.mat-background-accent .mat-tab-link:not(.mat-tab-disabled):focus, .mat-tab-nav-bar.mat-background-accent .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-nav-bar.mat-background-accent .mat-tab-link:not(.mat-tab-disabled):focus {\n     background-color: rgba(179, 224, 219, 0.3); }\n\n   .mat-tab-group.mat-background-accent .mat-tab-header, .mat-tab-group.mat-background-accent .mat-tab-links, .mat-tab-nav-bar.mat-background-accent .mat-tab-header, .mat-tab-nav-bar.mat-background-accent .mat-tab-links {\n     background-color: #009688; }\n\n   .mat-tab-group.mat-background-accent .mat-tab-label, .mat-tab-group.mat-background-accent .mat-tab-link, .mat-tab-nav-bar.mat-background-accent .mat-tab-label, .mat-tab-nav-bar.mat-background-accent .mat-tab-link {\n     color: white; }\n     .mat-tab-group.mat-background-accent .mat-tab-label.mat-tab-disabled, .mat-tab-group.mat-background-accent .mat-tab-link.mat-tab-disabled, .mat-tab-nav-bar.mat-background-accent .mat-tab-label.mat-tab-disabled, .mat-tab-nav-bar.mat-background-accent .mat-tab-link.mat-tab-disabled {\n       color: rgba(255, 255, 255, 0.4); }\n\n   .mat-tab-group.mat-background-accent .mat-tab-header-pagination-chevron, .mat-tab-nav-bar.mat-background-accent .mat-tab-header-pagination-chevron {\n     border-color: white; }\n\n   .mat-tab-group.mat-background-accent .mat-tab-header-pagination-disabled .mat-tab-header-pagination-chevron, .mat-tab-nav-bar.mat-background-accent .mat-tab-header-pagination-disabled .mat-tab-header-pagination-chevron {\n     border-color: rgba(255, 255, 255, 0.4); }\n\n   .mat-tab-group.mat-background-accent .mat-ripple-element, .mat-tab-nav-bar.mat-background-accent .mat-ripple-element {\n     background-color: rgba(255, 255, 255, 0.12); }\n\n   .mat-tab-group.mat-background-warn .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-group.mat-background-warn .mat-tab-link:not(.mat-tab-disabled):focus, .mat-tab-nav-bar.mat-background-warn .mat-tab-label:not(.mat-tab-disabled):focus,\n   .mat-tab-nav-bar.mat-background-warn .mat-tab-link:not(.mat-tab-disabled):focus {\n     background-color: rgba(255, 205, 210, 0.3); }\n\n   .mat-tab-group.mat-background-warn .mat-tab-header, .mat-tab-group.mat-background-warn .mat-tab-links, .mat-tab-nav-bar.mat-background-warn .mat-tab-header, .mat-tab-nav-bar.mat-background-warn .mat-tab-links {\n     background-color: #f44336; }\n\n   .mat-tab-group.mat-background-warn .mat-tab-label, .mat-tab-group.mat-background-warn .mat-tab-link, .mat-tab-nav-bar.mat-background-warn .mat-tab-label, .mat-tab-nav-bar.mat-background-warn .mat-tab-link {\n     color: white; }\n     .mat-tab-group.mat-background-warn .mat-tab-label.mat-tab-disabled, .mat-tab-group.mat-background-warn .mat-tab-link.mat-tab-disabled, .mat-tab-nav-bar.mat-background-warn .mat-tab-label.mat-tab-disabled, .mat-tab-nav-bar.mat-background-warn .mat-tab-link.mat-tab-disabled {\n       color: rgba(255, 255, 255, 0.4); }\n\n   .mat-tab-group.mat-background-warn .mat-tab-header-pagination-chevron, .mat-tab-nav-bar.mat-background-warn .mat-tab-header-pagination-chevron {\n     border-color: white; }\n\n   .mat-tab-group.mat-background-warn .mat-tab-header-pagination-disabled .mat-tab-header-pagination-chevron, .mat-tab-nav-bar.mat-background-warn .mat-tab-header-pagination-disabled .mat-tab-header-pagination-chevron {\n     border-color: rgba(255, 255, 255, 0.4); }\n\n   .mat-tab-group.mat-background-warn .mat-ripple-element, .mat-tab-nav-bar.mat-background-warn .mat-ripple-element {\n     background-color: rgba(255, 255, 255, 0.12); }\n\n   .mat-toolbar {\n     background: whitesmoke;\n     color: rgba(0, 0, 0, 0.87); }\n     .mat-toolbar.mat-primary {\n       background: #009688;\n       color: white; }\n     .mat-toolbar.mat-accent {\n       background: #009688;\n       color: white; }\n     .mat-toolbar.mat-warn {\n       background: #f44336;\n       color: white; }\n\n   .mat-tooltip {\n     background: rgba(97, 97, 97, 0.9); }\n\n   .mat-snack-bar-container {\n     background: #323232;\n     color: white; }\n\n   .mat-simple-snackbar-action {\n     color: #009688; }\n\n   .openvidu-appearin {\n     display: -webkit-box;\n     display: -webkit-flex;\n     display: -ms-flexbox;\n     display: flex;\n     -webkit-box-orient: vertical;\n     -webkit-box-direction: normal;\n     -webkit-flex-direction: column;\n         -ms-flex-direction: column;\n             flex-direction: column; }\n     .openvidu-appearin.fullscreen {\n       width: 100%;\n       height: 100%; }\n     .openvidu-appearin #welcome {\n       -webkit-box-flex: 1;\n       -webkit-flex: 1;\n           -ms-flex: 1;\n               flex: 1;\n       text-align: center;\n       color: #000;\n       margin: 5em auto auto;\n       max-width: 412px;\n       padding: 1em; }\n       .openvidu-appearin #welcome header {\n         margin-bottom: 18px; }\n         .openvidu-appearin #welcome header h1 {\n           margin: 0;\n           font-weight: 400;\n           line-height: 28px;\n           font-size: 20px; }\n       .openvidu-appearin #welcome section {\n         padding: 0 0 2em 0; }\n         .openvidu-appearin #welcome section .ai-spinner {\n           height: 120px;\n           width: 120px;\n           margin: 0 auto;\n           padding: 15px;\n           -webkit-transform-origin: 50% 50%;\n               -ms-transform-origin: 50% 50%;\n                   transform-origin: 50% 50%;\n           -webkit-animation: spin .7s linear infinite;\n                   animation: spin .7s linear infinite;\n           -webkit-box-sizing: border-box;\n                   box-sizing: border-box; }\n         .openvidu-appearin #welcome section .text-state {\n           font-size: 24px; }\n         .openvidu-appearin #welcome section .text-body-main {\n           font-size: 18px; }\n         .openvidu-appearin #welcome section .ui-button {\n           background: #ff2e63; }\n         .openvidu-appearin #welcome section .ui-button, .openvidu-appearin #welcome section .ui-button-invis, .openvidu-appearin #welcome section .ui-button-outline {\n           display: inline-block;\n           -webkit-border-radius: 3px;\n                   border-radius: 3px;\n           border: solid 0;\n           -webkit-box-shadow: 0 1px 0 0 rgba(0, 0, 0, 0.15);\n                   box-shadow: 0 1px 0 0 rgba(0, 0, 0, 0.15);\n           cursor: pointer;\n           font-size: 14px;\n           padding: 10px 12px 10px 12px;\n           line-height: 20px;\n           text-decoration: none;\n           position: relative;\n           text-align: center;\n           margin-top: 4px;\n           margin-bottom: 4px;\n           outline: 0;\n           font-weight: 400;\n           word-wrap: break-word;\n           border: solid #ff2e63 0;\n           color: #fff;\n           -webkit-font-smoothing: subpixel-antialiased;\n           -moz-osx-font-smoothing: auto; }\n         .openvidu-appearin #welcome section .video-preview-wrapper {\n           height: 256px;\n           background: rgba(0, 0, 0, 0.1);\n           overflow: hidden;\n           display: -webkit-box;\n           display: -webkit-flex;\n           display: -ms-flexbox;\n           display: flex;\n           -webkit-box-align: center;\n           -webkit-align-items: center;\n               -ms-flex-align: center;\n                   align-items: center;\n           -webkit-box-pack: center;\n           -webkit-justify-content: center;\n               -ms-flex-pack: center;\n                   justify-content: center;\n           -webkit-box-orient: vertical;\n           -webkit-box-direction: normal;\n           -webkit-flex-direction: column;\n               -ms-flex-direction: column;\n                   flex-direction: column; }\n         .openvidu-appearin #welcome section .camera-off-icon mat-icon {\n           height: 84px;\n           width: 500px; }\n           .openvidu-appearin #welcome section .camera-off-icon mat-icon .fill {\n             fill: #333; }\n     .openvidu-appearin .feedback-wrapper {\n       -webkit-box-flex: 1;\n       -webkit-flex: 1;\n           -ms-flex: 1;\n               flex: 1;\n       text-align: center;\n       background-color: rgba(0, 0, 0, 0.5);\n       padding: 50px;\n       -webkit-border-radius: 3px;\n               border-radius: 3px; }\n       .openvidu-appearin .feedback-wrapper .thank-you-message {\n         font-size: 34px;\n         margin-top: .5em;\n         margin-bottom: 1.5em;\n         color: #fff; }\n       .openvidu-appearin .feedback-wrapper p button {\n         background: #41cc8d; }\n     .openvidu-appearin .wrapper {\n       -webkit-box-flex: 1;\n       -webkit-flex: 1;\n           -ms-flex: 1;\n               flex: 1;\n       display: -webkit-box;\n       display: -webkit-flex;\n       display: -ms-flexbox;\n       display: flex;\n       -webkit-box-orient: vertical;\n       -webkit-box-direction: normal;\n       -webkit-flex-direction: column;\n           -ms-flex-direction: column;\n               flex-direction: column;\n       background: -webkit-gradient(linear, left top, left bottom, from(#360033), to(#0b8793));\n       background: -webkit-linear-gradient(#360033, #0b8793);\n       background: -o-linear-gradient(#360033, #0b8793);\n       background: linear-gradient(#360033, #0b8793); }\n       .openvidu-appearin .wrapper .toolbar .mat-toolbar {\n         background: transparent;\n         color: #fff; }\n       .openvidu-appearin .wrapper .toolbar .example-icon {\n         padding: 0 14px; }\n       .openvidu-appearin .wrapper .toolbar .example-spacer {\n         -webkit-box-flex: 1;\n         -webkit-flex: 1 1 auto;\n             -ms-flex: 1 1 auto;\n                 flex: 1 1 auto; }\n       .openvidu-appearin .wrapper mat-sidenav-container {\n         -webkit-box-flex: 1;\n         -webkit-flex: 1;\n             -ms-flex: 1;\n                 flex: 1;\n         background: transparent; }\n         .openvidu-appearin .wrapper mat-sidenav-container .chat-button {\n           position: absolute;\n           right: 12px;\n           bottom: 2.1em;\n           border: 2px solid #fff;\n           -webkit-transition: opacity .2s;\n           -o-transition: opacity .2s;\n           transition: opacity .2s;\n           -webkit-border-radius: 50%;\n                   border-radius: 50%;\n           background-color: rgba(0, 0, 0, 0.4);\n           cursor: pointer;\n           outline: 0;\n           margin: 0;\n           color: #fff;\n           z-index: 99; }\n         .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content {\n           display: -webkit-box;\n           display: -webkit-flex;\n           display: -ms-flexbox;\n           display: flex;\n           -webkit-box-orient: vertical;\n           -webkit-box-direction: normal;\n           -webkit-flex-direction: column;\n               -ms-flex-direction: column;\n                   flex-direction: column;\n           height: 100%; }\n           .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos {\n             -webkit-box-flex: 1;\n             -webkit-flex: 1;\n                 -ms-flex: 1;\n                     flex: 1;\n             display: -webkit-box;\n             display: -webkit-flex;\n             display: -ms-flexbox;\n             display: flex;\n             min-height: 200px; }\n             .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .stream-flex {\n               -webkit-box-flex: 1;\n               -webkit-flex: 1;\n                   -ms-flex: 1;\n                       flex: 1;\n               display: -webkit-box;\n               display: -webkit-flex;\n               display: -ms-flexbox;\n               display: flex;\n               -webkit-box-orient: horizontal;\n               -webkit-box-direction: normal;\n               -webkit-flex-direction: row;\n                   -ms-flex-direction: row;\n                       flex-direction: row;\n               -webkit-align-content: center;\n                   -ms-flex-line-pack: center;\n                       align-content: center;\n               -webkit-flex-wrap: wrap;\n                   -ms-flex-wrap: wrap;\n                       flex-wrap: wrap;\n               -webkit-box-pack: center;\n               -webkit-justify-content: center;\n                   -ms-flex-pack: center;\n                       justify-content: center; }\n             .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .chat {\n               display: -webkit-box;\n               display: -webkit-flex;\n               display: -ms-flexbox;\n               display: flex;\n               -webkit-box-orient: vertical;\n               -webkit-box-direction: normal;\n               -webkit-flex-direction: column;\n                   -ms-flex-direction: column;\n                       flex-direction: column;\n               -webkit-box-pack: end;\n               -webkit-justify-content: flex-end;\n                   -ms-flex-pack: end;\n                       justify-content: flex-end;\n               position: relative;\n               background-color: rgba(0, 0, 0, 0.5);\n               -webkit-border-radius: 3px 3px 0 0;\n                       border-radius: 3px 3px 0 0;\n               margin: 0px 10px;\n               width: 30%;\n               max-width: 400px; }\n               .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .chat .close {\n                 background: rgba(0, 0, 0, 0.4);\n                 color: #fff;\n                 outline: none;\n                 border: none;\n                 padding: 4px;\n                 -webkit-border-radius: 50%;\n                         border-radius: 50%;\n                 width: 36px;\n                 position: absolute;\n                 top: 0px;\n                 right: 0px;\n                 cursor: pointer; }\n                 .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .chat .close:hover {\n                   background-color: #000; }\n               .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .chat ul {\n                 -webkit-box-flex: 1;\n                 -webkit-flex: 1;\n                     -ms-flex: 1;\n                         flex: 1;\n                 padding: 0px 10px 10px 10px;\n                 overflow: auto; }\n                 .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .chat ul li {\n                   display: -webkit-box;\n                   display: -webkit-flex;\n                   display: -ms-flexbox;\n                   display: flex;\n                   -webkit-box-orient: horizontal;\n                   -webkit-box-direction: normal;\n                   -webkit-flex-direction: row;\n                       -ms-flex-direction: row;\n                           flex-direction: row;\n                   -webkit-box-pack: start;\n                   -webkit-justify-content: flex-start;\n                       -ms-flex-pack: start;\n                           justify-content: flex-start;\n                   padding: 6px 14px; }\n                   .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .chat ul li.me {\n                     -webkit-box-orient: horizontal;\n                     -webkit-box-direction: reverse;\n                     -webkit-flex-direction: row-reverse;\n                         -ms-flex-direction: row-reverse;\n                             flex-direction: row-reverse; }\n                     .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .chat ul li.me p {\n                       background-color: #ffcad7; }\n                   .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .chat ul li p {\n                     background-color: #f5fcf9;\n                     margin: 0;\n                     margin-right: 20px;\n                     padding: 7px 8px 6px;\n                     font-size: 12px;\n                     line-height: 1.3333;\n                     background-color: #f5fcf9;\n                     -webkit-border-radius: 3px;\n                             border-radius: 3px;\n                     word-wrap: break-word;\n                     max-width: -webkit-calc(100% - 68px);\n                     max-width: calc(100% - 68px);\n                     color: rgba(0, 0, 0, 0.87); }\n               .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .chat .message-input-container {\n                 background: #fff;\n                 padding: 0px 10px; }\n                 .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .panel-videos .chat .message-input-container mat-input-container {\n                   width: 100%; }\n           .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar {\n             display: -webkit-box;\n             display: -webkit-flex;\n             display: -ms-flexbox;\n             display: flex;\n             margin: 4px 0px 4px 0px; }\n             .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons {\n               background-color: rgba(0, 0, 0, 0.5);\n               -webkit-border-radius: 3px;\n                       border-radius: 3px;\n               display: -webkit-box;\n               display: -webkit-flex;\n               display: -ms-flexbox;\n               display: flex;\n               -webkit-box-pack: justify;\n               -webkit-justify-content: space-between;\n                   -ms-flex-pack: justify;\n                       justify-content: space-between;\n               margin: 0 auto;\n               overflow: hidden;\n               -webkit-box-align: center;\n               -webkit-align-items: center;\n                   -ms-flex-align: center;\n                       align-items: center; }\n               .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button {\n                 background-color: transparent;\n                 -webkit-box-align: center;\n                 -webkit-align-items: center;\n                     -ms-flex-align: center;\n                         align-items: center;\n                 -webkit-box-sizing: border-box;\n                         box-sizing: border-box;\n                 color: #fff;\n                 display: -webkit-box;\n                 display: -webkit-flex;\n                 display: -ms-flexbox;\n                 display: flex;\n                 -webkit-box-orient: vertical;\n                 -webkit-box-direction: normal;\n                 -webkit-flex-flow: column nowrap;\n                     -ms-flex-flow: column nowrap;\n                         flex-flow: column nowrap;\n                 font-size: 12px;\n                 height: 54px;\n                 -webkit-box-pack: center;\n                 -webkit-justify-content: center;\n                     -ms-flex-pack: center;\n                         justify-content: center;\n                 padding: 0 8px;\n                 text-align: center;\n                 width: 92px; }\n                 .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button.close-session {\n                   background-color: #ff2e63;\n                   height: 46px;\n                   width: 46px;\n                   margin: 0 16px; }\n                 .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button.disabled {\n                   background-color: #ff2e63; }\n                   .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button.disabled mat-icon {\n                     display: none; }\n                     .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button.disabled mat-icon.disabled-icon {\n                       display: inline-block; }\n                   .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button.disabled div {\n                     display: none; }\n                     .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button.disabled div.disabled-text {\n                       display: block; }\n                 .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button mat-icon {\n                   display: inline-block; }\n                   .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button mat-icon.disabled-icon {\n                     display: none; }\n                 .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button div {\n                   margin-bottom: 4px;\n                   text-shadow: 0 1px 1px rgba(0, 0, 0, 0.4);\n                   white-space: nowrap;\n                   width: 100%;\n                   line-height: 10px; }\n                   .openvidu-appearin .wrapper mat-sidenav-container .mat-sidenav-content .bottom-bar .buttons button div.disabled-text {\n                     display: none; }\n\n   @-webkit-keyframes spin {\n     from {\n       -webkit-transform: rotate(0);\n               transform: rotate(0); }\n     to {\n       -webkit-transform: rotate(360deg);\n               transform: rotate(360deg); } }\n\n   @keyframes spin {\n     from {\n       -webkit-transform: rotate(0);\n               transform: rotate(0); }\n     to {\n       -webkit-transform: rotate(360deg);\n               transform: rotate(360deg); } }\n\t"]
        }),
        __metadata("design:paramtypes", [core_1.Renderer2, angular_bigscreen_1.BigScreenService,
            openvidu_appearin_intl_1.OpenViduAppearinIntl])
    ], OpenViduAppearinComponent);
    return OpenViduAppearinComponent;
}(openvidu_internal_component_1.OpenViduInternalComponent));
exports.OpenViduAppearinComponent = OpenViduAppearinComponent;
